#    Copyright (c) 2019-2023 IDEMIA
#    Author: IDEMIA (Philippe Fremy, Florent Oulieres)
# 
#     Licensed under the Apache License, Version 2.0 (the "License");
#     you may not use this file except in compliance with the License.
#     You may obtain a copy of the License at
# 
#         http://www.apache.org/licenses/LICENSE-2.0
# 
#     Unless required by applicable law or agreed to in writing, software
#     distributed under the License is distributed on an "AS IS" BASIS,
#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#     See the License for the specific language governing permissions and
#     limitations under the License.
#


from typing import List, Dict, Callable, Any

import enum

from PySide2.QtCore import QObject, Signal, Qt
from PySide2.QtWidgets import QWidget, QProgressDialog, QApplication

from src.mg_repo_info import MgRepoInfo



class RepoInfoFlags(enum.Flag):
    HEAD = enum.auto()
    URL = enum.auto()
    SHA1 = enum.auto()
    ALL_TAGS = enum.auto()
    ALL_BRANCHES = enum.auto()
    DIFF_SUMMARY = enum.auto()
    FILES_SHA1 = enum.auto()


class MgEnsureInfoAvailable(QObject):

    sigInfoAvailable = Signal()

    def __init__(self, parent: QWidget, repoList: List['MgRepoInfo'], showProgressDialog: bool = False) -> None:
        super().__init__(parent)
        self.progressDialog = None
        if showProgressDialog:
            self.progressDialog = QProgressDialog(parent)
            self.progressDialog.setCancelButton(None)
            self.progressDialog.setMinimum(0)
            self.progressDialog.setMinimumDuration(1000)    # 1s before showing the dialog
            self.progressDialog.setWindowTitle('Progress')
            self.progressDialog.setLabelText('Collecting repository information')
            self.progressDialog.setWindowFlags(self.progressDialog.windowFlags() & ~Qt.WindowContextHelpButtonHint & ~Qt.WindowCloseButtonHint)
            self.progressDialog.setMinimumWidth(300)
            self.progressDialog.setMinimumHeight(50)
        self.repoList = repoList

        # number of repo where information is expected/available
        self.infoAvailable = {} # type: Dict[RepoInfoFlags, int]
        self.infoExpected = {}  # type: Dict[RepoInfoFlags, int]
        self.clear()


    def clear(self) -> None:
        '''Clear the dictionnaries tracking the number of information expected and available'''
        for infoType in list(RepoInfoFlags):
            self.infoAvailable[infoType] = 0
            self.infoExpected[infoType] = 0


    def ensureInfoAvailable(self, infoTypes: 'RepoInfoFlags', blocking: bool = False) -> None:
        '''Ensure that the requested information is available in all listed repos.

        blocking: True - do not return before all information is available
                   False - return immediately and emit a completed() signal when all info is avaialble.
        '''
        self.clear()
        if len(self.repoList) == 0 or infoTypes.value == 0:
            # sometimes, we request information from an empty list or we reqest empty information

            # ensure progress dialog does not show up
            if self.progressDialog:
                self.progressDialog.setMaximum(1)
                self.progressDialog.setValue(1)

            self.sigInfoAvailable.emit()
            return

        steps = 0
        for infoFlag in list(RepoInfoFlags):
            if infoTypes & infoFlag:
                self.infoExpected[infoFlag] = len(self.repoList)
                steps += len(self.repoList)

        if self.progressDialog:
            self.progressDialog.setMaximum(steps)
            self.progressDialog.setValue(0)
            # self.progressDialog.show()
            QApplication.processEvents()

        # collect HEAD + URL at the same time
        # The code is kind of weird because we always call the same function. The trick is that since genCallbackInfoAvailable()
        # uses the number of info fields to advance the steps toward completion, if we request only URL but use genCallbackInfoAvailable()
        # with 2 fields, it will advance by 2 steps and mess the count
        if (RepoInfoFlags.HEAD in infoTypes) or (RepoInfoFlags.URL in infoTypes):
            flagsToFill: RepoInfoFlags
            if (RepoInfoFlags.HEAD in infoTypes) and (RepoInfoFlags.URL in infoTypes):
                flagsToFill = RepoInfoFlags.HEAD | RepoInfoFlags.URL
            elif (RepoInfoFlags.HEAD in infoTypes):
                flagsToFill = RepoInfoFlags.HEAD
            else:
                flagsToFill = RepoInfoFlags.URL
            for repo in self.repoList:
                repo.ensure_head_and_url(blocking, self.genCallbackInfoAvailable(flagsToFill))
                QApplication.processEvents()

        if RepoInfoFlags.SHA1 in infoTypes:
            for repo in self.repoList:
                repo.ensure_sha1(self.genCallbackInfoAvailable(RepoInfoFlags.SHA1), blocking)
                QApplication.processEvents()

        if RepoInfoFlags.ALL_TAGS in infoTypes:
            for repo in self.repoList:
                repo.ensure_all_tags_filled(self.genCallbackInfoAvailable(RepoInfoFlags.ALL_TAGS), blocking)
                QApplication.processEvents()

        if RepoInfoFlags.DIFF_SUMMARY in infoTypes:
            for repo in self.repoList:
                repo.ensure_diff_summary(self.genCallbackInfoAvailable(RepoInfoFlags.DIFF_SUMMARY), blocking)
                QApplication.processEvents()

        if RepoInfoFlags.ALL_BRANCHES in infoTypes:
            for repo in self.repoList:
                repo.ensure_branches_filled(self.genCallbackInfoAvailable(RepoInfoFlags.ALL_BRANCHES), blocking)
                QApplication.processEvents()

        if RepoInfoFlags.FILES_SHA1 in infoTypes:
            for repo in self.repoList:
                repo.ensure_files_sha1_filled(self.genCallbackInfoAvailable(RepoInfoFlags.FILES_SHA1), blocking)
                QApplication.processEvents()


    def genCallbackInfoAvailable(self, repoInfos: RepoInfoFlags) -> Callable[..., None]:
        '''Return a function which will call self.repoInfoAvailable() with the repoInfos argument.

        To be used to generate a callback .
        '''
        def callbackRepoInfoAvailable(*args: Any) -> None:
            self.repoInfoAvailable(repoInfos)

        return callbackRepoInfoAvailable


    def repoInfoAvailable(self, infoTypes: 'RepoInfoFlags') -> None:
        '''Notify that the given information is available on a new repo.

        Emit sigInfoAvailable() if all information expected is now available.
        '''
        for infoFlag in list(RepoInfoFlags):
            if infoTypes & infoFlag:
                self.infoAvailable[infoFlag] += 1
                if self.progressDialog:
                    self.progressDialog.setValue(self.progressDialog.value() + 1)
                    QApplication.processEvents()  # give a chance to asynchronous jobs to complete + refresh dialog
        self.emitSigInfoAvailableIfDone()


    def emitSigInfoAvailableIfDone(self) -> None:
        if self.isAllInfoAvailable():
            self.sigInfoAvailable.emit()


    def isAllInfoAvailable(self) -> bool:
        '''Return True if all expectations are met'''
        for infoFlag in list(RepoInfoFlags):
            if self.infoExpected[infoFlag] != self.infoAvailable[infoFlag]:
                return False

        return True


#    Copyright (c) 2019-2023 IDEMIA
#    Author: IDEMIA (Philippe Fremy, Florent Oulieres)
# 
#     Licensed under the Apache License, Version 2.0 (the "License");
#     you may not use this file except in compliance with the License.
#     You may obtain a copy of the License at
# 
#         http://www.apache.org/licenses/LICENSE-2.0
# 
#     Unless required by applicable law or agreed to in writing, software
#     distributed under the License is distributed on an "AS IS" BASIS,
#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#     See the License for the specific language governing permissions and
#     limitations under the License.
#


from typing import Sequence, cast, Any, Tuple, List
import logging, time

from PySide2.QtCore import Qt, QTimer
from PySide2.QtGui import QGuiApplication
from PySide2.QtWidgets import QDialog, QTreeWidgetItem, QDialogButtonBox, QWidget, QApplication, QPushButton

from src.gui.ui_git_exec_window import Ui_GitExecDialog
import src.mg_config as mgc
from src.mg_repo_info import MgRepoInfo
from src.mg_auth_failure_mgr import MgAuthFailureMgr
from src.mg_exec_task_item import MgExecTaskGit, MgExecTask, MgExecItemBase, MgExecItemOneCmd, MgExecItemMultiCmd, MgExecTaskGroup, after_other_taskgroup_is_finished, PreConditionState

logger = logging.getLogger('mg_git_exec_window')
dbg = logger.debug
warning = logger.warning
error = logger.error


# If you get git stack-traces / crash looking like:
# 0 [main] sh (88240) C:\Program Files\Git\usr\bin\sh.exe: *** fatal error - add_item ("\??\C:\Program Files\Git", "/", ...) failed, errno 1
# Increase this time
#
# Also, a delay is needed to avoid that when user changed his password and the stored password is no longer valid
# we don't want too many git process to be started before the first processes fail and a proper action is proposed
# to the user.
DELTA_BETWEEN_CONCURRENT_GIT_RUN = 0.10

INDENT_TEXT = '    '
def collectColumnText(depth: int, item: QTreeWidgetItem) -> List[str]:
    '''Collect text of item and all child, with an indentation'''
    result = [ INDENT_TEXT * depth + text for text in item.text(0).split('\n') ]
    for itemChildIdx in range(item.childCount()):
        result.extend( collectColumnText(depth + 1, item.child(itemChildIdx) ) )
    return result


# noinspection PyAttributeOutsideInit
class MgExecWindow(QDialog):
    '''Window for the execution of one or several git commands'''

    def __init__(self, parent: QWidget, delta_between_concurrent_git_run: float = DELTA_BETWEEN_CONCURRENT_GIT_RUN) -> None:
        super().__init__(parent)
        self.setModal(False)
        # noinspection PyTypeChecker
        self.setWindowFlags(self.windowFlags() & ~Qt.WindowContextHelpButtonHint)
        self.ui = Ui_GitExecDialog()
        self.ui.setupUi(self)
        self.ui.treeGitJobs.itemExpanded.connect(self.autoAdjustColumnSize)
        self.ui.buttonBox.button(QDialogButtonBox.Discard).clicked.connect( self.slotAbort )
        self.ui.buttonBox.button(QDialogButtonBox.Discard).setText('Abort')
        self.buttonCopyLog = QPushButton('Copy git log to clipboard', self.ui.buttonBox)
        self.ui.buttonBox.addButton(self.buttonCopyLog, QDialogButtonBox.ActionRole)
        self.ui.textEditSummary.hide()
        self.buttonCopyLog.clicked.connect( self.slotCopyLog )
        self.delta_between_concurrent_git_run = delta_between_concurrent_git_run
        self.clear()
        MgAuthFailureMgr.newSession()

    def clear(self) -> None:
        self.abort_requested = False
        self.ui.treeGitJobs.clear()
        self.nb_errors = 0
        self.nb_jobs = 0
        self.nb_jobs_running = 0
        self.nb_jobs_done = 0
        self.last_started_job_time = 0.0
        self.first_started_job_time = 0.0
        self.duration = 0.0
        self.askQuestionUponFailure = True


    def execOneGitCommand(self, desc: str, cmdline: List[str], repos: Sequence[MgRepoInfo],
                          display_window: bool = True) -> None:
        '''Execute the same git command on each repo of the list of repos.

        cmdline: command-line to execute on the repo, provided as a list of string.
                 The git command should not be part of it.
        '''
        return self.execSeqOfGitCommand(desc, [cmdline], repos, display_window)


    def execSeqOfGitCommand(self, desc: str, cmdlineSeq: List[List[str]], repos: Sequence[MgRepoInfo],
                            display_window: bool = True) -> None:
        '''Run the sequence of git commands on each repo listed in repos

        desc: a string used to describe the command under execution
        cmdlineSeq: list of command-lines for git as a list of string for the git command.
                    The git command should not be part of it.
        repos: a sequence of MgRepoInfo to run the command on

        This will:
        - show the window
        - execute the command on each repo in background, updating the window live
        - display errors in red
        - once command is completed, refresh the item in the view
        - update the progress bar
        - show a final status
        '''
        self.execEachRepoWithHisSeqOfGitCommand(desc, [(repo.name, repo, cmdlineSeq) for repo in repos], display_window)


    def execEachRepoWithHisSeqOfGitCommand(self, globalDesc: str,
                                           cmdlinePerRepo: List[Tuple[str, MgRepoInfo, List[List[str]]]],
                                           display_window: bool = True) -> None:
        '''For each repo, execute a different sequence of git command.

        globalDesc: a global description string
        cmdLinePerRepo: a Tuple containing the following elements:
            * a description string of the command to run
            * the repo on which to run the git command
            * the sequence git command to execute, as a list of list of string, excluding the git executable itself

        Example:
            execEachRepoWithHisSeqOfGitCommand('Cloning Repositories', [
                ('cloning multigit from GitHub',
                 repoInfo,
                 [
                    ['clone', 'https://github.com/Multigit', 'c:\\work\\MultiGit' ],
                    ['checkout', 'master'],
                 ]
                ),
                ('cloning multigit plugins',
                 repoInfo,
                 [
                    ['clone', 'https://git.oberthur.com/Multigit-plugin', 'c:\\work\\MultiGit-plugin'],
                    ['checkout', 'master'],
                 ]
                ),
                ...
            ])
        '''
        taskGroups = [
            MgExecTaskGroup(repo.name, repo, [cast(MgExecTask, MgExecTaskGit('', repo, cmdLine)) for cmdLine in cmdlineSeq])
            for (desc, repo, cmdlineSeq) in cmdlinePerRepo
        ]
        # tasksPerRepo[0][2][0].pre_condition = precond_other_task_finished(tasksPerRepo[-1][2][0])
        self.execTaskGroups(globalDesc, taskGroups, display_window)


    def execTaskGroups(self, globalDesc: str,
                       taskGroups: List[MgExecTaskGroup],
                       display_window: bool = True
                       ) -> None:
        '''Execute each task group under a global description'''
        dbg(f'execTaskGroups({globalDesc}, {taskGroups}, {display_window})')
        if display_window is True:
            self.show()
        self.nb_jobs += len(taskGroups)
        self.abort_requested = False

        tli = QTreeWidgetItem([globalDesc])
        self.ui.treeGitJobs.addTopLevelItem( tli )
        tli.setExpanded(True)

        jobitem: MgExecItemBase
        for taskGroup in taskGroups:
            # multiple jobs to run, even if there is only one item
            jobitem = MgExecItemMultiCmd(taskGroup, self.oneMoreJobDone, self.askQuestionUponFailure)
            tli.addChild( jobitem )

        # 1/3 for starting the job
        # +2/3 for job completion
        # we set a goal to 1 when there are zero jobs
        self.ui.progressBar.setMaximum(max(self.nb_jobs*3, 1))
        self.autoAdjustColumnSize()

        self.duration = 0.0
        self.first_started_job_time = time.time()
        self.updateProgress()

        self.last_started_job_time = time.time() - self.delta_between_concurrent_git_run - 0.1 # note: force the delta to run the first job
        self.startNewJobs()


    def startNewJobs(self) -> None:
        dbg('startNewJobs')
        if self.abort_requested:
            dbg('startNewJobs() - Avoid starting new jobs because abort was requested')
            return

        nb_jobs_started = 0
        nb_jobs_blocked_by_precondition = 0
        max_git_process = mgc.get_config_instance().get(mgc.CONFIG_NB_GIT_PROC, 0)
        for jobItem in self.getAllJobItems():
            if max_git_process and self.nb_jobs_running == max_git_process:
                dbg('startNewJobs() - max number of process reached')
                # we have reach our maximum
                return

            dbg('startNewJobs() - looking at job to start: %s' % jobItem)
            if not jobItem.isStarted:
                if jobItem.taskGroup.is_precondition_fulfilled() == PreConditionState.NotFulfilled :
                    dbg(f'startNewjobs() - condition not fullfilled for {jobItem.taskGroup}, choosing the next one')
                    nb_jobs_blocked_by_precondition += 1
                    continue

                if jobItem.taskGroup.is_precondition_fulfilled() == PreConditionState.Errored :
                    dbg(f'startNewjobs() - condition already in error for {jobItem.taskGroup}, aborting the item')
                    jobItem.abortItem()
                    continue

                # condition is fulfilled, we can start

                # leave some time between each run of process, else you get a git crash
                # this is a git/cygwin concurrency issue so limit the concurrency here
                # also, it is better visually.
                cur_time = time.time()
                while cur_time - self.last_started_job_time < self.delta_between_concurrent_git_run:
                    QApplication.processEvents()
                    cur_time = time.time()

                # note: the processEvent() call may catch another job finishing, triggering
                #       a call to startNewJobs() in parallel to the currently running one.
                #       This in-between call may start the job which we are currently looking at.
                #       So be sure to check again that the job is not started.

                if not jobItem.isStarted:
                    dbg('startNewJobs() - actually starting: %s' % jobItem)
                    self.last_started_job_time = cur_time
                    self.nb_jobs_running += 1
                    nb_jobs_started += 1
                    jobItem.run()
                else:
                    # job was started in the interim
                    dbg('startNewJobs() - job started while we were waiting: {}'.format(jobItem))
                    dbg('startNewJobs() - proceed to the next job')

        if nb_jobs_blocked_by_precondition > 0 and not self.abort_requested \
                and (not max_git_process or self.nb_jobs_running < max_git_process):
            # we have room to start new jobs, but we did not do it, certainly because of non fulfilled
            # preconditions. In this case, let's give some delay for the preconditions and call back this
            # method again.
            # If we don't to this, we rely on job completion to call startNewJobs(), but some precondition
            # can fulfill before any job completion.
            dbg('startNewJobs() - give a chance to start jobs again in one second')
            QTimer.singleShot(1000, self.startNewJobs)
            return


    def getAllJobItems(self) -> List[MgExecItemMultiCmd]:
        '''Return a list of all MgExecItemMultiCmd'''
        all_job_items: List[MgExecItemMultiCmd] = []
        for tliIdx in range(self.ui.treeGitJobs.topLevelItemCount()):
            tli = self.ui.treeGitJobs.topLevelItem(tliIdx)
            assert tli is not None
            for childIdx in range(tli.childCount()):
                jobItem = cast(MgExecItemMultiCmd, tli.child(childIdx))
                assert isinstance(jobItem, MgExecItemMultiCmd)
                all_job_items.append(jobItem)
        return all_job_items


    def updateProgress(self) -> None:
        if self.nb_errors > 0:
            msg = '%d of %d completed, with %d errors' % (self.nb_jobs_done, self.nb_jobs, self.nb_errors)
        else:
            msg = '%d of %d completed' % (self.nb_jobs_done, self.nb_jobs)
        if self.nb_jobs_done == self.nb_jobs and self.duration != 0.0:
            msg += ' in %0.1f seconds' % self.duration
        self.ui.labelTasks.setText(msg)
        if self.nb_jobs == 0:
            # when running with no jobs, we fake the 100% immediatly
            self.ui.progressBar.setValue(1)
        else:
            self.ui.progressBar.setValue(self.nb_jobs + self.nb_jobs_done*2)
        completed = (self.nb_jobs_done == self.nb_jobs)
        self.ui.buttonBox.button(QDialogButtonBox.Discard).setEnabled(not completed)
        self.ui.buttonBox.button(QDialogButtonBox.Ok).setEnabled(completed)


    def oneMoreJobDone(self, success: bool) -> None:
        '''Called when one job is completed, with the success status'''
        dbg(f'oneMoreJobDone(success={success})')
        self.nb_jobs_done += 1
        self.nb_jobs_running -= 1
        if self.nb_jobs_done == self.nb_jobs:
            # we are done !
            self.duration = time.time() - self.first_started_job_time
        if not success:
            self.nb_errors += 1
        dbg('Running %d, done %d, errors %d, remaining %d' % (self.nb_jobs_running, self.nb_jobs_done,
            self.nb_errors, self.nb_jobs-self.nb_jobs_done))
        self.updateProgress()
        self.autoAdjustColumnSize()

        if not self.abort_requested and self.nb_jobs_done < self.nb_jobs:
            # we have not started all our jobs (probably because of the MAX_PROCESS limitation),
            # start the remaining ones
            dbg(f'Starting new jobs for {self.nb_jobs - self.nb_jobs_done} jobs remaining')
            self.startNewJobs()


    def slotAbort(self) -> None:
        '''Triggered when abort is requested'''
        dbg('Aborting...')
        self.abort_requested = True
        for topLevelIdx in range(self.ui.treeGitJobs.topLevelItemCount()-1, -1, -1):
            topLevel = self.ui.treeGitJobs.topLevelItem(topLevelIdx)
            assert topLevel is not None
            # note: we must abort in the reverse order, else one abort is going to trigger the next job
            #       which is not aborted yet at the item level
            for i in reversed(range(topLevel.childCount())):
                childItem = topLevel.child(i)
                if isinstance(childItem, MgExecItemBase):
                    childItem.abortItem()


    def slotCopyLog(self) -> None:
        '''Copy the log of the window to the clipboard'''
        fullLog = []        # type: List[str]
        for topLevelIdx in range(0, self.ui.treeGitJobs.topLevelItemCount()):
            item = self.ui.treeGitJobs.topLevelItem(topLevelIdx)
            assert item is not None
            fullLog.extend( collectColumnText(0, item) )

        QGuiApplication.clipboard().setText('\n'.join(fullLog))


    def autoAdjustColumnSize(self) -> None:
        '''Adjust automatically the column size to the largest item'''
        for i in range(self.ui.treeGitJobs.columnCount()):
            self.ui.treeGitJobs.resizeColumnToContents(i)
        QApplication.processEvents()


    def done(self, v: Any) -> None:
        '''Called by the OK button when closing the dialog'''
        dbg('done()')
        self.clear()
        super().done(v)


    def waitForJobCompletion(self) -> None:
        '''Wait until all running jobs have completed'''
        while self.nb_jobs_done < self.nb_jobs:
            QApplication.processEvents()


# TODO: ensure that passing multiple commands per repo gets a proper title


#    Copyright (c) 2019-2023 IDEMIA
#    Author: IDEMIA (Philippe Fremy, Florent Oulieres)
# 
#     Licensed under the Apache License, Version 2.0 (the "License");
#     you may not use this file except in compliance with the License.
#     You may obtain a copy of the License at
# 
#         http://www.apache.org/licenses/LICENSE-2.0
# 
#     Unless required by applicable law or agreed to in writing, software
#     distributed under the License is distributed on an "AS IS" BASIS,
#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#     See the License for the specific language governing permissions and
#     limitations under the License.
#
from typing import Union, List, Optional
import os, sys
import argparse, sys, os
from pathlib import Path

if __package__:
    # when running inside a package, src/* is not on the path, only multigit_gx is.
    # we must add it explicitely
    path = os.path.dirname(__file__)
    sys.path.insert(0, path)

from src import mg_const as mgc
from src.mg_json_mgit_parser import ProjectStructure
from src.mg_tools import RunProcess, ExecGit

HELP = '''
mgitcmd clone path_to_mgit_file.mgit [--dest path_to_dir] [--shallow] 
    Clone a group of repositories according to path_to_mgit_file.mgit

    --dest path_to_dir: Clones into the target directory. If not provided,
                        clones into the current directory.

    --shalllow: Perform a shallow clone with --depth 1 . Applied only on tags
                and branches, will not work for HEAD specified as a commit. This
                speeds up the clone process by downloading less data.


mgitcmd --version
    Displays version information and exits.


mgitcmd --help
    Display this help

'''

VERSION_LINE = 'Multigit Command-line version v%s' % mgc.VERSION


def cmd_clone(fname: str, dest_dir: Optional[str], shallow: bool) -> None:
    if not Path(fname).exists():
        print('Mgit file does not exist: %s' % fname)
        sys.exit(0)
    project = ProjectStructure()
    base_dir = Path(dest_dir or '.').resolve()
    project.fill_from_json_file(fname, base_dir)
    print('Cloning information:')
    print(project)

    reposToClone = project.repos[:]
    reposToClone.sort(key=lambda r: r.destination)

    errors = []
    tasks: List[Union[str, List[str]]] = []
    for repo in reposToClone:
        if Path(repo.dest_fullpath).exists():
            errors.append('Directory already exists, can not clone: %s' % repo.dest_fullpath)
        if repo.url == '':
            errors.append('Can not clone repository with empty URL: %s' % repo.destination)
        tasks.append('Cloning ' + repo.destination)
        clone_cmd = ['clone', repo.url, repo.dest_fullpath]
        tasks.append(clone_cmd)
        if shallow and repo.head_type in ('branch', 'tag'):
            clone_cmd.extend(['--branch', repo.head])
            clone_cmd.extend(['--depth', '1'])
        else:
            # head_type: commit or empty
            if len(repo.head):
                # if head is empty, this is an empty repository, nothing to checkout
                tasks.append('Checkout %s@%s' % (repo.destination, repo.head))
                # use the '--' to make sure this is a branch/commit checkout
                tasks.append(['-C', repo.dest_fullpath, 'checkout', '-q', repo.head, '--'])

    if errors:
        print('Errors:')
        print('- ' + '\n- '.join(errors))
        print('Aborting')
        sys.exit(-1)

    prog_git = ExecGit.get_executable()
    if prog_git is None or len(prog_git) == 0:
        print('Can not find git with executable!')
        print('Please put it on the path or define git location in the Multigit settings.')
        sys.exit(-1)

    for task in tasks:
        if type(task) is str:
            print('[[ %s ]]' % task)
            continue

        git_cmd = [prog_git, *task]

        # our pool process does not handle force blocking, launch directly
        print('> ' + ' '.join(git_cmd))
        exit_code, cmd_out = RunProcess().exec_blocking(git_cmd, allow_errors=True)
        print(cmd_out)

        if exit_code != 0:
            print('Git failure (exit code %d)' % exit_code)
            sys.exit(-1)

    print('Clone successful.')


def main() -> None:
    if '--help' in sys.argv or len(sys.argv) == 1:
        print('USAGE: ' + HELP)
        sys.exit(0)

    if '--version' in sys.argv:
        print(VERSION_LINE)
        sys.exit(0)

    parser = argparse.ArgumentParser(add_help=False, usage=HELP)
    parser.add_argument('clone', nargs=2)
    parser.add_argument('--dest', nargs=1)
    parser.add_argument('--shallow', action='store_true', default=False)
    parse_result = parser.parse_args()

    print(VERSION_LINE)
    cmd_clone(parse_result.clone[1], (parse_result.dest or [None])[0], parse_result.shallow)

if __name__ == '__main__':
    main()





#    Copyright (c) 2019-2023 IDEMIA
#    Author: IDEMIA (Philippe Fremy, Florent Oulieres)
# 
#     Licensed under the Apache License, Version 2.0 (the "License");
#     you may not use this file except in compliance with the License.
#     You may obtain a copy of the License at
# 
#         http://www.apache.org/licenses/LICENSE-2.0
# 
#     Unless required by applicable law or agreed to in writing, software
#     distributed under the License is distributed on an "AS IS" BASIS,
#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#     See the License for the specific language governing permissions and
#     limitations under the License.
#


from typing import List, cast, Optional

from PySide2.QtWidgets import QDialog, QWidget, QTreeWidget
from PySide2.QtCore import Qt, QSize
from PySide2.QtGui import QIcon

from src.gui.ui_select_repos import Ui_SelectRepos
from src.mg_repo_info import MgRepoInfo
from src.mg_repo_tree_item import MgRepoTreeItem
from src.mg_dialog_utils import prepareTreeWidgetRepoList
from src.mg_const import *


class MgDialogSelectRepos(QDialog):
    '''Select a list of repositories from a given set of repositories'''

    def __init__(self, parent: QWidget, selectedRepoList: List[MgRepoInfo],
                 allRepoList: List[MgRepoInfo]) -> None:
        super().__init__(parent)

        # noinspection PyTypeChecker
        self.setWindowFlags(self.windowFlags() & ~Qt.WindowContextHelpButtonHint)
        self.ui = Ui_SelectRepos()
        self.ui.setupUi(self)
        prepareTreeWidgetRepoList(self.ui.treeWidgetSelectedRepos)
        prepareTreeWidgetRepoList(self.ui.treeWidgetAvailRepos)

        firstItem = True
        for repoInfo in selectedRepoList:
            repoTreeItem = MgRepoTreeItem(repoInfo, self.ui.treeWidgetSelectedRepos)
            repoTreeItem.fillRepoItem()
            if firstItem:
                repoTreeItem.setSelected(True)
                firstItem = False


        firstItem = True
        for repoInfo in allRepoList:
            if repoInfo in selectedRepoList:
                continue
            repoTreeItem = MgRepoTreeItem(repoInfo, self.ui.treeWidgetAvailRepos)
            repoTreeItem.fillRepoItem()
            if firstItem:
                repoTreeItem.setSelected(True)
                firstItem = False

        MgRepoTreeItem.autoAdjustColumnSize(self.ui.treeWidgetSelectedRepos)
        MgRepoTreeItem.autoAdjustColumnSize(self.ui.treeWidgetAvailRepos)

        self.ui.pushButtonMoveUp.setIconSize(QSize(32, 32))
        self.ui.pushButtonMoveDown.setIconSize(QSize(32, 32))
        self.ui.pushButtonMoveAllDown.setIconSize(QSize(32, 32))
        self.ui.pushButtonMoveAllUp.setIconSize(QSize(32, 32))
        self.ui.pushButtonMoveUp.setIcon(QIcon(':img/icons8-chevron-up-52.png'))
        self.ui.pushButtonMoveDown.setIcon(QIcon(':img/icons8-chevron-down-52.png'))
        self.ui.pushButtonMoveAllDown.setIcon(QIcon(':img/icons8-double-down-52.png'))
        self.ui.pushButtonMoveAllUp.setIcon(QIcon(':img/icons8-double-up-52.png'))
        self.ui.pushButtonMoveUp.setText('')
        self.ui.pushButtonMoveDown.setText('')
        self.ui.pushButtonMoveAllUp.setText('')
        self.ui.pushButtonMoveAllDown.setText('')
        self.ui.pushButtonMoveUp.clicked.connect(self.slotMoveToAvailable)
        self.ui.pushButtonMoveDown.clicked.connect(self.slotMoveToTargeted)
        self.ui.pushButtonMoveAllUp.clicked.connect(self.slotMoveAllToAvailable)
        self.ui.pushButtonMoveAllDown.clicked.connect(self.slotMoveAllToTargeted)
        self.ui.pushButtonMoveDown.setToolTip('Move selected repositories to targeted list')
        self.ui.pushButtonMoveUp.setToolTip('Remove selected repository from targeted list')
        self.ui.pushButtonMoveAllDown.setToolTip('Move all repositories to targeted list')
        self.ui.pushButtonMoveAllUp.setToolTip('Remove all repositories from targeted list')
        self.updateLabelSelectedRepoNb()

        self.ui.treeWidgetAvailRepos.setFocus()

        # we must freeze the size of the labelLineHidden because else, it keeps
        # changing size and make the lineEditFilter widget move
        self.ui.labelLineHidden.setText(' ' * len('XXX repositories hidden'))
        self.ui.labelLineHidden.setFixedSize(self.ui.labelLineHidden.size())
        self.ui.labelLineHidden.setText('')

        self.ui.lineEditFilterList.textEdited.connect(self.showHideRepoFromFilter)


    def showHideRepoFromFilter(self, text: Optional[str]) -> None:
        '''Called when the filter is updated, hide/show the relevant items'''
        text = text or self.ui.lineEditFilterList.text()
        text = text.lower()
        nbHidden = 0
        for idx in range(self.ui.treeWidgetAvailRepos.topLevelItemCount()):
            item = self.ui.treeWidgetAvailRepos.topLevelItem(idx)
            assert item is not None
            item.setHidden(True)
            nbHidden += 1
            for i in range(COL_NB):
                if text in item.text(i).lower():
                    item.setHidden(False)
                    nbHidden -= 1
                    break

        if nbHidden == 0 and text == '':
            self.ui.labelLineHidden.setText('')
        else:
            self.ui.labelLineHidden.setText('%d repositories hidden' % nbHidden)


    def selectedRepoInfo(self) -> List[MgRepoInfo]:
        '''Return the list of chosen repositories'''
        return [cast(MgRepoTreeItem, self.ui.treeWidgetSelectedRepos.topLevelItem(idx)).repoInfo
                for idx in range(self.ui.treeWidgetSelectedRepos.topLevelItemCount())]


    def slotMoveAllToAvailable(self) -> None:
        '''Move item seleccted to the treeWidgetAvailableRepos'''
        self.moveItemsToOtherTreeWidget(self.ui.treeWidgetSelectedRepos, self.ui.treeWidgetAvailRepos,
                                        True)


    def slotMoveToAvailable(self) -> None:
        '''Move item seleccted to the treeWidgetAvailableRepos'''
        self.moveItemsToOtherTreeWidget(self.ui.treeWidgetSelectedRepos, self.ui.treeWidgetAvailRepos)


    def slotMoveAllToTargeted(self) -> None:
        '''Move item seleccted to the treeWidgetSelectedRepos'''
        self.moveItemsToOtherTreeWidget(self.ui.treeWidgetAvailRepos, self.ui.treeWidgetSelectedRepos,
                                        True)


    def slotMoveToTargeted(self) -> None:
        '''Move item seleccted to the treeWidgetSelectedRepos'''
        self.moveItemsToOtherTreeWidget(self.ui.treeWidgetAvailRepos, self.ui.treeWidgetSelectedRepos)


    def moveItemsToOtherTreeWidget(self, treeWidgetSource: QTreeWidget,
                                         treeWidgetDest: QTreeWidget,
                                         moveAll: bool = False) -> None:
        '''Remove items from one treeWidget and create items on the other tree Widget'''
        if moveAll:
            treeWidgetSource.selectAll()
        selectedItems = treeWidgetSource.selectedItems()

        for item in selectedItems:
            # add item to destination
            assert isinstance(item, MgRepoTreeItem)
            repoTreeItem = MgRepoTreeItem(item.repoInfo, treeWidgetDest)
            repoTreeItem.fillRepoItem()
            if treeWidgetDest.topLevelItemCount() == 1:
                # we are moving our first item there, select it
                repoTreeItem.setSelected(True)

            # remove item from source
            idx = treeWidgetSource.indexOfTopLevelItem(item)
            treeWidgetSource.takeTopLevelItem(idx)

            # select another item after the move
            if treeWidgetSource.topLevelItemCount():
                # if there are still items to select

                # find a valid index
                idx =  min(idx, treeWidgetSource.topLevelItemCount()-1)
                treeWidgetSource.topLevelItem(idx).setSelected(True)    # type: ignore # topLevelItem(idx) exists and is not None

        self.updateLabelSelectedRepoNb()
        self.showHideRepoFromFilter(None)


    def updateLabelSelectedRepoNb(self) -> None:
        self.ui.labelRepoSelected.setText('%d repositories targeted' % self.ui.treeWidgetSelectedRepos.topLevelItemCount())






#    Copyright (c) 2019-2023 IDEMIA
#    Author: IDEMIA (Philippe Fremy, Florent Oulieres)
# 
#     Licensed under the Apache License, Version 2.0 (the "License");
#     you may not use this file except in compliance with the License.
#     You may obtain a copy of the License at
# 
#         http://www.apache.org/licenses/LICENSE-2.0
# 
#     Unless required by applicable law or agreed to in writing, software
#     distributed under the License is distributed on an "AS IS" BASIS,
#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#     See the License for the specific language governing permissions and
#     limitations under the License.
#


from typing import List
import logging

from PySide2.QtWidgets import QMessageBox, QWidget

from src.mg_exec_window import MgExecWindow
from src.mg_dialog_utils import MgDialogWithRepoList
from src.gui.ui_git_run_command import Ui_GitRunCommand
from src import mg_config as mgc
from src.mg_const import *
from src.mg_repo_info import MgRepoInfo

logger = logging.getLogger('mg_dialog_git_cmd')
dbg = logger.debug


class MgDialogGitRunCmd(MgDialogWithRepoList):
    ui: Ui_GitRunCommand

    def __init__(self, parent: QWidget, selectedRepos: List[MgRepoInfo], allRepos: List[MgRepoInfo]) -> None:
        super().__init__(parent, Ui_GitRunCommand, selectedRepos, allRepos)
        self.fillMenuHistory()
        self.ui.historyButton.historyItemTriggered.connect(self.slotHistoryItemTriggered)


    def slotHistoryItemTriggered(self, title: str, message: str) -> None:
        '''Triggered when an item from the history menu has been selected'''
        self.ui.lineEditGitCmd.setText(message)


    def fillMenuHistory(self) -> None:
        config = mgc.get_config_instance()

        # Fill up list of recent git commands
        self.ui.historyButton.fillHistoryWithTitleAndContent(
            [(f'git {gitCmd}', gitCmd) for gitCmd in config.lruAsList(mgc.CONFIG_GIT_CMD_HISTORY)]
        )


    def accept(self) -> None:
        gitCmd = self.ui.lineEditGitCmd.text()
        if len(gitCmd) == 0:
            QMessageBox.warning(self, "Empty git command", "You dit not specify any git command. Operation canceled!")
            return

        # archive git command
        config = mgc.get_config_instance()
        config.lruSetRecent(mgc.CONFIG_GIT_CMD_HISTORY, gitCmd)
        config.save()

        super().accept()


def runDialogGitCommand(parent: QWidget, selectedRepos: List[MgRepoInfo], allRepos: List[MgRepoInfo]) -> None:
    '''Run a custom git command'''
    dbg('runGitCommand')

    dlg = MgDialogGitRunCmd(parent, selectedRepos, allRepos)
    result = dlg.exec()
    if not result:
        # command execution canceled
        return

    # perform run git command
    gitCmdText = dlg.ui.lineEditGitCmd.text()
    desc = 'Running git command: git ' + gitCmdText
    gitCmd = gitCmdText.split(' ')

    gitExecWindow = MgExecWindow(parent)
    gitExecWindow.execOneGitCommand(desc, gitCmd, dlg.getTargetedRepoList())




# -*- coding: utf-8 -*-

#    Copyright (c) 2019-2023 IDEMIA
#    Author: IDEMIA (Philippe Fremy, Florent Oulieres)
# 
#     Licensed under the Apache License, Version 2.0 (the "License");
#     you may not use this file except in compliance with the License.
#     You may obtain a copy of the License at
# 
#         http://www.apache.org/licenses/LICENSE-2.0
# 
#     Unless required by applicable law or agreed to in writing, software
#     distributed under the License is distributed on an "AS IS" BASIS,
#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#     See the License for the specific language governing permissions and
#     limitations under the License.
#


################################################################################
## Form generated from reading UI file 'ui_main_window.ui'
##
## Created by: Qt User Interface Compiler version 5.15.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *

# import multigit_resources_rc

class Ui_MainWindow(object):
    def setupUi(self, MainWindow: QMainWindow) -> None:
        if not MainWindow.objectName():
            MainWindow.setObjectName(u"MainWindow")
        MainWindow.resize(290, 625)
        self.actionOpenDirectory = QAction(MainWindow)
        self.actionOpenDirectory.setObjectName(u"actionOpenDirectory")
        icon = QIcon()
        icon.addFile(u":/img/icons8-open-folder-64.png", QSize(), QIcon.Normal, QIcon.Off)
        self.actionOpenDirectory.setIcon(icon)
        self.actionOpenDirectory.setShortcutContext(Qt.ApplicationShortcut)
        self.actionEditPreferences = QAction(MainWindow)
        self.actionEditPreferences.setObjectName(u"actionEditPreferences")
        self.actionEditPreferences.setShortcutContext(Qt.ApplicationShortcut)
        self.actionQuit = QAction(MainWindow)
        self.actionQuit.setObjectName(u"actionQuit")
        self.actionQuit.setShortcutContext(Qt.ApplicationShortcut)
        self.actionRefreshAll = QAction(MainWindow)
        self.actionRefreshAll.setObjectName(u"actionRefreshAll")
        self.actionDir1 = QAction(MainWindow)
        self.actionDir1.setObjectName(u"actionDir1")
        self.actionDir2 = QAction(MainWindow)
        self.actionDir2.setObjectName(u"actionDir2")
        self.actionAbout = QAction(MainWindow)
        self.actionAbout.setObjectName(u"actionAbout")
        self.actionWhatIsNew = QAction(MainWindow)
        self.actionWhatIsNew.setObjectName(u"actionWhatIsNew")
        self.actionExportCSV = QAction(MainWindow)
        self.actionExportCSV.setObjectName(u"actionExportCSV")
        self.actionOpenProject = QAction(MainWindow)
        self.actionOpenProject.setObjectName(u"actionOpenProject")
        self.actionExportToMgit = QAction(MainWindow)
        self.actionExportToMgit.setObjectName(u"actionExportToMgit")
        self.actionShowMultiGitLogFiles = QAction(MainWindow)
        self.actionShowMultiGitLogFiles.setObjectName(u"actionShowMultiGitLogFiles")
        self.actionViewLastCommit = QAction(MainWindow)
        self.actionViewLastCommit.setObjectName(u"actionViewLastCommit")
        self.actionViewLastCommit.setCheckable(True)
        self.actionViewLastCommit.setChecked(True)
        self.actionViewModifiedFiles = QAction(MainWindow)
        self.actionViewModifiedFiles.setObjectName(u"actionViewModifiedFiles")
        self.actionViewModifiedFiles.setCheckable(True)
        self.actionViewColSha1 = QAction(MainWindow)
        self.actionViewColSha1.setObjectName(u"actionViewColSha1")
        self.actionViewColSha1.setCheckable(True)
        self.actionApplyMultigitFile = QAction(MainWindow)
        self.actionApplyMultigitFile.setObjectName(u"actionApplyMultigitFile")
        self.actionViewColURL = QAction(MainWindow)
        self.actionViewColURL.setObjectName(u"actionViewColURL")
        self.actionViewColURL.setCheckable(True)
        self.actionAddTab = QAction(MainWindow)
        self.actionAddTab.setObjectName(u"actionAddTab")
        self.actionDupTab = QAction(MainWindow)
        self.actionDupTab.setObjectName(u"actionDupTab")
        self.actionCloseTab = QAction(MainWindow)
        self.actionCloseTab.setObjectName(u"actionCloseTab")
        self.actionRenameTab = QAction(MainWindow)
        self.actionRenameTab.setObjectName(u"actionRenameTab")
        self.centralwidget = QWidget(MainWindow)
        self.centralwidget.setObjectName(u"centralwidget")
        self.horizontalLayout_2 = QHBoxLayout(self.centralwidget)
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.tabRepos = QTabWidget(self.centralwidget)
        self.tabRepos.setObjectName(u"tabRepos")
        self.tabRepos.setMovable(True)

        self.horizontalLayout_2.addWidget(self.tabRepos)

        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QMenuBar(MainWindow)
        self.menubar.setObjectName(u"menubar")
        self.menubar.setGeometry(QRect(0, 0, 290, 21))
        self.menuFile = QMenu(self.menubar)
        self.menuFile.setObjectName(u"menuFile")
        self.menuFile.setToolTipsVisible(True)
        self.menuOpenRecentDirectory = QMenu(self.menuFile)
        self.menuOpenRecentDirectory.setObjectName(u"menuOpenRecentDirectory")
        self.menuOpenRecentDirectory.setToolTipsVisible(True)
        self.menuView = QMenu(self.menubar)
        self.menuView.setObjectName(u"menuView")
        self.menuView.setToolTipsVisible(True)
        self.menuShow = QMenu(self.menuView)
        self.menuShow.setObjectName(u"menuShow")
        self.menuColumns = QMenu(self.menuShow)
        self.menuColumns.setObjectName(u"menuColumns")
        self.menuGit = QMenu(self.menubar)
        self.menuGit.setObjectName(u"menuGit")
        self.menuGit.setToolTipsVisible(True)
        self.menuGitPrograms = QMenu(self.menubar)
        self.menuGitPrograms.setObjectName(u"menuGitPrograms")
        self.menuGitPrograms.setToolTipsVisible(True)
        self.menuAbout = QMenu(self.menubar)
        self.menuAbout.setObjectName(u"menuAbout")
        self.menuAbout.setToolTipsVisible(True)
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QStatusBar(MainWindow)
        self.statusbar.setObjectName(u"statusbar")
        MainWindow.setStatusBar(self.statusbar)

        self.menubar.addAction(self.menuFile.menuAction())
        self.menubar.addAction(self.menuView.menuAction())
        self.menubar.addAction(self.menuGit.menuAction())
        self.menubar.addAction(self.menuGitPrograms.menuAction())
        self.menubar.addAction(self.menuAbout.menuAction())
        self.menuFile.addAction(self.actionOpenDirectory)
        self.menuFile.addAction(self.menuOpenRecentDirectory.menuAction())
        self.menuFile.addAction(self.actionEditPreferences)
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.actionAddTab)
        self.menuFile.addAction(self.actionDupTab)
        self.menuFile.addAction(self.actionRenameTab)
        self.menuFile.addAction(self.actionCloseTab)
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.actionOpenProject)
        self.menuFile.addAction(self.actionApplyMultigitFile)
        self.menuFile.addAction(self.actionExportToMgit)
        self.menuFile.addAction(self.actionExportCSV)
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.actionQuit)
        self.menuOpenRecentDirectory.addAction(self.actionDir1)
        self.menuOpenRecentDirectory.addAction(self.actionDir2)
        self.menuView.addAction(self.menuShow.menuAction())
        self.menuView.addAction(self.actionRefreshAll)
        self.menuShow.addAction(self.actionViewLastCommit)
        self.menuShow.addAction(self.actionViewModifiedFiles)
        self.menuShow.addAction(self.menuColumns.menuAction())
        self.menuColumns.addAction(self.actionViewColSha1)
        self.menuColumns.addAction(self.actionViewColURL)
        self.menuAbout.addAction(self.actionWhatIsNew)
        self.menuAbout.addAction(self.actionShowMultiGitLogFiles)
        self.menuAbout.addAction(self.actionAbout)

        self.retranslateUi(MainWindow)

        self.tabRepos.setCurrentIndex(-1)


        QMetaObject.connectSlotsByName(MainWindow)
    # setupUi

    def retranslateUi(self, MainWindow: QMainWindow) -> None:
        MainWindow.setWindowTitle(QCoreApplication.translate("MainWindow", u"MultiGit", None))
        self.actionOpenDirectory.setText(QCoreApplication.translate("MainWindow", u"Open base directory", None))
#if QT_CONFIG(tooltip)
        self.actionOpenDirectory.setToolTip(QCoreApplication.translate("MainWindow", u"Open base directory", None))
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(shortcut)
        self.actionOpenDirectory.setShortcut(QCoreApplication.translate("MainWindow", u"Ctrl+O", None))
#endif // QT_CONFIG(shortcut)
        self.actionEditPreferences.setText(QCoreApplication.translate("MainWindow", u"Edit settings", None))
#if QT_CONFIG(tooltip)
        self.actionEditPreferences.setToolTip(QCoreApplication.translate("MainWindow", u"Edit settings", None))
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(shortcut)
        self.actionEditPreferences.setShortcut(QCoreApplication.translate("MainWindow", u"Ctrl+S", None))
#endif // QT_CONFIG(shortcut)
        self.actionQuit.setText(QCoreApplication.translate("MainWindow", u"Quit", None))
        self.actionQuit.setIconText(QCoreApplication.translate("MainWindow", u"Quit", None))
#if QT_CONFIG(tooltip)
        self.actionQuit.setToolTip(QCoreApplication.translate("MainWindow", u"Quit", None))
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(shortcut)
        self.actionQuit.setShortcut(QCoreApplication.translate("MainWindow", u"Ctrl+X", None))
#endif // QT_CONFIG(shortcut)
        self.actionRefreshAll.setText(QCoreApplication.translate("MainWindow", u"Refresh local view", None))
#if QT_CONFIG(tooltip)
        self.actionRefreshAll.setToolTip(QCoreApplication.translate("MainWindow", u"Refresh repository information from disk", None))
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(shortcut)
        self.actionRefreshAll.setShortcut(QCoreApplication.translate("MainWindow", u"F5", None))
#endif // QT_CONFIG(shortcut)
        self.actionDir1.setText(QCoreApplication.translate("MainWindow", u"Dir1", None))
        self.actionDir2.setText(QCoreApplication.translate("MainWindow", u"Dir2", None))
        self.actionAbout.setText(QCoreApplication.translate("MainWindow", u"About MultiGit", None))
        self.actionWhatIsNew.setText(QCoreApplication.translate("MainWindow", u"What's new", None))
#if QT_CONFIG(tooltip)
        self.actionWhatIsNew.setToolTip(QCoreApplication.translate("MainWindow", u"Show dialog with What's New information", None))
#endif // QT_CONFIG(tooltip)
        self.actionExportCSV.setText(QCoreApplication.translate("MainWindow", u"Export to CSV", None))
#if QT_CONFIG(tooltip)
        self.actionExportCSV.setToolTip(QCoreApplication.translate("MainWindow", u"Export repository informations to a CSV files", None))
#endif // QT_CONFIG(tooltip)
        self.actionOpenProject.setText(QCoreApplication.translate("MainWindow", u"Clone from Multigit file", None))
#if QT_CONFIG(tooltip)
        self.actionOpenProject.setToolTip(QCoreApplication.translate("MainWindow", u"Clone repositories described in a Multigit file", None))
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(shortcut)
        self.actionOpenProject.setShortcut(QCoreApplication.translate("MainWindow", u"Ctrl+J", None))
#endif // QT_CONFIG(shortcut)
        self.actionExportToMgit.setText(QCoreApplication.translate("MainWindow", u"Export to Multigit file", None))
#if QT_CONFIG(tooltip)
        self.actionExportToMgit.setToolTip(QCoreApplication.translate("MainWindow", u"Export repositories setup to a Multigit file", None))
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(shortcut)
        self.actionExportToMgit.setShortcut(QCoreApplication.translate("MainWindow", u"Ctrl+Shift+J", None))
#endif // QT_CONFIG(shortcut)
        self.actionShowMultiGitLogFiles.setText(QCoreApplication.translate("MainWindow", u"Open directory of log files", None))
#if QT_CONFIG(tooltip)
        self.actionShowMultiGitLogFiles.setToolTip(QCoreApplication.translate("MainWindow", u"Open directory containing log files and configuration", None))
#endif // QT_CONFIG(tooltip)
        self.actionViewLastCommit.setText(QCoreApplication.translate("MainWindow", u"Show Last Commit tab", None))
        self.actionViewModifiedFiles.setText(QCoreApplication.translate("MainWindow", u"Show Modified Files tab", None))
        self.actionViewColSha1.setText(QCoreApplication.translate("MainWindow", u"Sha1", None))
        self.actionApplyMultigitFile.setText(QCoreApplication.translate("MainWindow", u"Apply Multigit file", None))
#if QT_CONFIG(tooltip)
        self.actionApplyMultigitFile.setToolTip(QCoreApplication.translate("MainWindow", u"Adjust existing repositories according to content of Multigit file", None))
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(shortcut)
        self.actionApplyMultigitFile.setShortcut(QCoreApplication.translate("MainWindow", u"Ctrl+Shift+A", None))
#endif // QT_CONFIG(shortcut)
        self.actionViewColURL.setText(QCoreApplication.translate("MainWindow", u"Remote URL", None))
        self.actionAddTab.setText(QCoreApplication.translate("MainWindow", u"New tab", None))
#if QT_CONFIG(shortcut)
        self.actionAddTab.setShortcut(QCoreApplication.translate("MainWindow", u"Ctrl+N", None))
#endif // QT_CONFIG(shortcut)
        self.actionDupTab.setText(QCoreApplication.translate("MainWindow", u"Duplicate tab", None))
#if QT_CONFIG(shortcut)
        self.actionDupTab.setShortcut(QCoreApplication.translate("MainWindow", u"Ctrl+D", None))
#endif // QT_CONFIG(shortcut)
        self.actionCloseTab.setText(QCoreApplication.translate("MainWindow", u"Close current tab", None))
#if QT_CONFIG(shortcut)
        self.actionCloseTab.setShortcut(QCoreApplication.translate("MainWindow", u"Ctrl+W", None))
#endif // QT_CONFIG(shortcut)
        self.actionRenameTab.setText(QCoreApplication.translate("MainWindow", u"Rename tab", None))
#if QT_CONFIG(shortcut)
        self.actionRenameTab.setShortcut(QCoreApplication.translate("MainWindow", u"F2", None))
#endif // QT_CONFIG(shortcut)
        self.menuFile.setTitle(QCoreApplication.translate("MainWindow", u"File", None))
        self.menuOpenRecentDirectory.setTitle(QCoreApplication.translate("MainWindow", u"Open recent directory", None))
        self.menuView.setTitle(QCoreApplication.translate("MainWindow", u"View", None))
        self.menuShow.setTitle(QCoreApplication.translate("MainWindow", u"Show", None))
        self.menuColumns.setTitle(QCoreApplication.translate("MainWindow", u"Show Columns", None))
        self.menuGit.setTitle(QCoreApplication.translate("MainWindow", u"Git", None))
        self.menuGitPrograms.setTitle(QCoreApplication.translate("MainWindow", u"Git Programs", None))
        self.menuAbout.setTitle(QCoreApplication.translate("MainWindow", u"About", None))
    # retranslateUi




#    Copyright (c) 2019-2023 IDEMIA
#    Author: IDEMIA (Philippe Fremy, Florent Oulieres)
# 
#     Licensed under the Apache License, Version 2.0 (the "License");
#     you may not use this file except in compliance with the License.
#     You may obtain a copy of the License at
# 
#         http://www.apache.org/licenses/LICENSE-2.0
# 
#     Unless required by applicable law or agreed to in writing, software
#     distributed under the License is distributed on an "AS IS" BASIS,
#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#     See the License for the specific language governing permissions and
#     limitations under the License.
#


from typing import TYPE_CHECKING, Any

if TYPE_CHECKING:
    from src.mg_window import MgMainWindow
    from src.mg_repo_tree import MgRepoTree
    from src.mg_dialog_clone_from_mgit import MgDialogCloneFromMgitFile


class MgPluginManager:

    def __init__(self) -> None:
        self.idemiaPlugin = None

    def registerIdemiaPlugin(self, idemiaPlugin: Any) -> None:
        self.idemiaPlugin = idemiaPlugin

    def setupTopMenu(self, window: 'MgMainWindow') -> None:
        '''Called during init of the actions and menus of the top window'''
        if not self.idemiaPlugin:
            return

        self.idemiaPlugin.setupTopMenu(window)

    def setupRepoRmbMenu(self, repoTree: 'MgRepoTree') -> None:
        '''Called when each MgRepoTree prepares his RMB menu'''
        if not self.idemiaPlugin:
            return

        self.idemiaPlugin.setupRepoRmbMenu(repoTree)


    def setupCloneDialog(self, cloneDialog: 'MgDialogCloneFromMgitFile') -> None:
        '''Called when preparing the clone Dialog'''
        if not self.idemiaPlugin:
            return

        self.idemiaPlugin.setupCloneDialog(cloneDialog)


pluginMgrInstance = MgPluginManager()

try:
    from idemia import idemia_plugin        # type: ignore[import-not-found]
    pluginMgrInstance.registerIdemiaPlugin(idemia_plugin.MgIdemiaPlugin())
except ImportError as exc:
    if not 'idemia' in str(exc):
        # this is an import error, but not for the idemia plugin
        # we should report it !
        raise






#    Copyright (c) 2019-2023 IDEMIA
#    Author: IDEMIA (Philippe Fremy, Florent Oulieres)
# 
#     Licensed under the Apache License, Version 2.0 (the "License");
#     you may not use this file except in compliance with the License.
#     You may obtain a copy of the License at
# 
#         http://www.apache.org/licenses/LICENSE-2.0
# 
#     Unless required by applicable law or agreed to in writing, software
#     distributed under the License is distributed on an "AS IS" BASIS,
#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#     See the License for the specific language governing permissions and
#     limitations under the License.
#


from typing import List

import logging

from PySide2.QtWidgets import QWidget, QDialogButtonBox

from src.mg_dialog_utils import MgDialogWithRepoList
from src.gui.ui_git_revert import Ui_GitRevert
from src.mg_exec_window import MgExecWindow
from src.mg_repo_info import MgRepoInfo
from src.mg_utils import htmlize_diff
from src.mg_ensure_info_available import MgEnsureInfoAvailable, RepoInfoFlags

logger = logging.getLogger('mg_dialog_git_revert')
dbg = logger.debug


class MgDialogGitRevert(MgDialogWithRepoList):
    ui: Ui_GitRevert

    def __init__(self, parent: QWidget, selectedRepos: List[MgRepoInfo], allRepos: List[MgRepoInfo]) -> None:
        super().__init__(parent, Ui_GitRevert, selectedRepos, allRepos)
        self.sigRepoListAdjusted.connect(self.updateDiffContent)
        self.updateDiffContent()
        buttonYes = self.ui.buttonBox.button( QDialogButtonBox.Yes )
        buttonYes.setText('Revert')


    def updateDiffContent(self) -> None:
        content = []
        self.ensureInfoAvailable = MgEnsureInfoAvailable(self, self.getTargetedRepoList(), showProgressDialog=True)
        self.ensureInfoAvailable.ensureInfoAvailable(RepoInfoFlags.DIFF_SUMMARY, blocking=True)
        for repo in self.getTargetedRepoList():
            # repo.ensure_diff_summary(blocking=True)
            if repo.diff_summary and len(repo.diff_summary):
                content.append(f'<b>{repo.name}:<b>')
                content.append( htmlize_diff(repo.diff_summary or '') + '<p>' )

        self.ui.textEditRevertContent.setText('\n'.join(content))



def runDialogGitRevert(parent: QWidget, selectedRepos: List[MgRepoInfo], allRepos: List[MgRepoInfo]) -> None:
    dbg('runDialogGitRevert')

    dlg = MgDialogGitRevert(parent, selectedRepos, allRepos)
    result = dlg.exec()
    if not result:
        # command execution canceled
        return

    desc = 'Reverting repositories'
    cmd = ['checkout', '--', '.']

    # show window for executing git
    gitExecWindow = MgExecWindow(parent)
    gitExecWindow.execOneGitCommand(desc, cmd, dlg.getTargetedRepoList())





# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'ui_git_create_branch.ui'
##
## Created by: Qt User Interface Compiler version 5.15.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *

from src.mg_repo_tree import MgRepoTree


class Ui_GitCreateBranch(object):
    def setupUi(self, GitCreateBranch: QDialog) -> None:
        if not GitCreateBranch.objectName():
            GitCreateBranch.setObjectName(u"GitCreateBranch")
        GitCreateBranch.resize(883, 423)
        self.verticalLayout_2 = QVBoxLayout(GitCreateBranch)
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.groupBox = QGroupBox(GitCreateBranch)
        self.groupBox.setObjectName(u"groupBox")
        self.gridLayout = QGridLayout(self.groupBox)
        self.gridLayout.setObjectName(u"gridLayout")
        self.horizontalSpacer_2 = QSpacerItem(554, 20, QSizePolicy.Preferred, QSizePolicy.Minimum)

        self.gridLayout.addItem(self.horizontalSpacer_2, 0, 3, 1, 1)

        self.checkBoxSwitchBranch = QCheckBox(self.groupBox)
        self.checkBoxSwitchBranch.setObjectName(u"checkBoxSwitchBranch")
        sizePolicy = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(3)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.checkBoxSwitchBranch.sizePolicy().hasHeightForWidth())
        self.checkBoxSwitchBranch.setSizePolicy(sizePolicy)
        self.checkBoxSwitchBranch.setChecked(True)

        self.gridLayout.addWidget(self.checkBoxSwitchBranch, 1, 1, 1, 1)

        self.label = QLabel(self.groupBox)
        self.label.setObjectName(u"label")
        self.label.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout.addWidget(self.label, 0, 0, 1, 1)

        self.comboBoxBranchName = QComboBox(self.groupBox)
        self.comboBoxBranchName.setObjectName(u"comboBoxBranchName")
        sizePolicy1 = QSizePolicy(QSizePolicy.MinimumExpanding, QSizePolicy.Fixed)
        sizePolicy1.setHorizontalStretch(3)
        sizePolicy1.setVerticalStretch(0)
        sizePolicy1.setHeightForWidth(self.comboBoxBranchName.sizePolicy().hasHeightForWidth())
        self.comboBoxBranchName.setSizePolicy(sizePolicy1)
        font = QFont()
        font.setFamily(u"Consolas")
        self.comboBoxBranchName.setFont(font)
        self.comboBoxBranchName.setEditable(True)

        self.gridLayout.addWidget(self.comboBoxBranchName, 0, 1, 1, 2)

        self.horizontalSpacer_3 = QSpacerItem(554, 20, QSizePolicy.Preferred, QSizePolicy.Minimum)

        self.gridLayout.addItem(self.horizontalSpacer_3, 1, 3, 1, 1)

        self.checkBoxPushToRemote = QCheckBox(self.groupBox)
        self.checkBoxPushToRemote.setObjectName(u"checkBoxPushToRemote")
        sizePolicy.setHeightForWidth(self.checkBoxPushToRemote.sizePolicy().hasHeightForWidth())
        self.checkBoxPushToRemote.setSizePolicy(sizePolicy)

        self.gridLayout.addWidget(self.checkBoxPushToRemote, 2, 1, 1, 1)


        self.verticalLayout_2.addWidget(self.groupBox)

        self.groupBox_2 = QGroupBox(GitCreateBranch)
        self.groupBox_2.setObjectName(u"groupBox_2")
        self.verticalLayout = QVBoxLayout(self.groupBox_2)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.labelRepoSelected = QLabel(self.groupBox_2)
        self.labelRepoSelected.setObjectName(u"labelRepoSelected")
        sizePolicy2 = QSizePolicy(QSizePolicy.Preferred, QSizePolicy.Preferred)
        sizePolicy2.setHorizontalStretch(1)
        sizePolicy2.setVerticalStretch(0)
        sizePolicy2.setHeightForWidth(self.labelRepoSelected.sizePolicy().hasHeightForWidth())
        self.labelRepoSelected.setSizePolicy(sizePolicy2)
        self.labelRepoSelected.setAlignment(Qt.AlignCenter)

        self.horizontalLayout.addWidget(self.labelRepoSelected)

        self.pushButtonAdjustRepoList = QPushButton(self.groupBox_2)
        self.pushButtonAdjustRepoList.setObjectName(u"pushButtonAdjustRepoList")
        sizePolicy3 = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Fixed)
        sizePolicy3.setHorizontalStretch(1)
        sizePolicy3.setVerticalStretch(0)
        sizePolicy3.setHeightForWidth(self.pushButtonAdjustRepoList.sizePolicy().hasHeightForWidth())
        self.pushButtonAdjustRepoList.setSizePolicy(sizePolicy3)

        self.horizontalLayout.addWidget(self.pushButtonAdjustRepoList)


        self.verticalLayout.addLayout(self.horizontalLayout)

        self.treeWidgetRepoList = MgRepoTree(self.groupBox_2)
        self.treeWidgetRepoList.setObjectName(u"treeWidgetRepoList")
        self.treeWidgetRepoList.setAlternatingRowColors(True)
        self.treeWidgetRepoList.setSelectionMode(QAbstractItemView.ExtendedSelection)
        self.treeWidgetRepoList.setRootIsDecorated(False)
        self.treeWidgetRepoList.setUniformRowHeights(True)
        self.treeWidgetRepoList.setItemsExpandable(False)
        self.treeWidgetRepoList.setSortingEnabled(True)

        self.verticalLayout.addWidget(self.treeWidgetRepoList)


        self.verticalLayout_2.addWidget(self.groupBox_2)

        self.horizontalLayout_4 = QHBoxLayout()
        self.horizontalLayout_4.setObjectName(u"horizontalLayout_4")
        self.buttonBox = QDialogButtonBox(GitCreateBranch)
        self.buttonBox.setObjectName(u"buttonBox")
        self.buttonBox.setOrientation(Qt.Horizontal)
        self.buttonBox.setStandardButtons(QDialogButtonBox.Cancel|QDialogButtonBox.Ok)

        self.horizontalLayout_4.addWidget(self.buttonBox)


        self.verticalLayout_2.addLayout(self.horizontalLayout_4)

        QWidget.setTabOrder(self.comboBoxBranchName, self.checkBoxSwitchBranch)
        QWidget.setTabOrder(self.checkBoxSwitchBranch, self.checkBoxPushToRemote)
        QWidget.setTabOrder(self.checkBoxPushToRemote, self.pushButtonAdjustRepoList)
        QWidget.setTabOrder(self.pushButtonAdjustRepoList, self.treeWidgetRepoList)

        self.retranslateUi(GitCreateBranch)
        self.buttonBox.accepted.connect(GitCreateBranch.accept)
        self.buttonBox.rejected.connect(GitCreateBranch.reject)

        QMetaObject.connectSlotsByName(GitCreateBranch)
    # setupUi

    def retranslateUi(self, GitCreateBranch: QDialog) -> None:
        GitCreateBranch.setWindowTitle(QCoreApplication.translate("GitCreateBranch", u"Git Create branch", None))
        self.groupBox.setTitle(QCoreApplication.translate("GitCreateBranch", u"Create Branch", None))
        self.checkBoxSwitchBranch.setText(QCoreApplication.translate("GitCreateBranch", u"Switch to branch after creation", None))
        self.label.setText(QCoreApplication.translate("GitCreateBranch", u"Enter branch name :", None))
        self.checkBoxPushToRemote.setText(QCoreApplication.translate("GitCreateBranch", u"Push branch to remote", None))
        self.groupBox_2.setTitle(QCoreApplication.translate("GitCreateBranch", u"Targeted repositories", None))
        self.labelRepoSelected.setText(QCoreApplication.translate("GitCreateBranch", u"3 repositories selected", None))
        self.pushButtonAdjustRepoList.setText(QCoreApplication.translate("GitCreateBranch", u"Adjust Repository list", None))
        ___qtreewidgetitem = self.treeWidgetRepoList.headerItem()
        ___qtreewidgetitem.setText(4, QCoreApplication.translate("GitCreateBranch", u"Last Remote Syncho", None));
        ___qtreewidgetitem.setText(3, QCoreApplication.translate("GitCreateBranch", u"Status", None));
        ___qtreewidgetitem.setText(2, QCoreApplication.translate("GitCreateBranch", u"Head", None));
        ___qtreewidgetitem.setText(1, QCoreApplication.translate("GitCreateBranch", u"Git Repo Path", None));
        ___qtreewidgetitem.setText(0, QCoreApplication.translate("GitCreateBranch", u"hidden", None));
    # retranslateUi




#    Copyright (c) 2019-2023 IDEMIA
#    Author: IDEMIA (Philippe Fremy, Florent Oulieres)
# 
#     Licensed under the Apache License, Version 2.0 (the "License");
#     you may not use this file except in compliance with the License.
#     You may obtain a copy of the License at
# 
#         http://www.apache.org/licenses/LICENSE-2.0
# 
#     Unless required by applicable law or agreed to in writing, software
#     distributed under the License is distributed on an "AS IS" BASIS,
#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#     See the License for the specific language governing permissions and
#     limitations under the License.
#


import pathlib, markdown2, datetime, sys

CONTENT_TXT_TEMPLATE = """
content_html = '''
%s
'''
"""


def markdown2qthtml(md_text: str) -> str:
    '''Transform a text in markdown syntax to a html subset supported by Qt

    Qt supports in the generated markdown:
    * <code> is put in fixed font
    * <pre> to show CR
    * <blockquote> to indent text

    Not supported
    * <code> does not have a grey background
    * multi-line code is not indented

    Transformations:
    * for multi-line code,
      * indent the text (blockquote),
      * make linebreaks visible in output (pre)
      * gray the code line (p style="...")
    * for words code
      * add grey background
    '''

    html_text: str
    html_text = markdown2.markdown(md_text)
    html_text = html_text.replace('<h1>', '<h1 align="center">')
    html_text = html_text.replace('<code>\n', '<p style="background-color:#f5f5f5;"><blockquote><pre><code>\n')
    html_text = html_text.replace('</code>\n', '</code></pre></blockquote></p>\n')
    html_text = html_text.replace('<code>', '<code><span style="background-color:#f5f5f5;">')
    html_text = html_text.replace('</code>', '</span></code>')
    return html_text


def markdown_to_python(source_md: pathlib.Path, dest_python: pathlib.Path) -> None:
    source_fname = source_md.name
    with open(source_md, encoding='utf8') as f:
        md_content = f.read()

    html_content = markdown2qthtml(md_content)

    with open(dest_python, 'w') as f:
        f.write(f'# Generated from {source_fname} on %s\n\n' % datetime.datetime.now().isoformat(' '))
        f.write(CONTENT_TXT_TEMPLATE % html_content)

if __name__ == '__main__':
    markdown_to_python(pathlib.Path(__file__).parent / 'CHANGELOG.md',
                       pathlib.Path(__file__).parent/'src/gui/content_whatisnew.py',
                       )

    markdown_to_python(pathlib.Path(__file__).parent / 'FULL_LICENSING_INFORMATION.md',
                       pathlib.Path(__file__).parent/'src/gui/content_full_license_info.py',
                       )

    if '--show' in sys.argv:
        from src.mg_dialog_whatisnew import showWhatIsNew
        from src.mg_dialog_about import showDialogAbout
        from PySide2.QtWidgets import QApplication
        _ = QApplication([])
        showWhatIsNew()
        showDialogAbout(None)







# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'ui_apply_mgit_file.ui'
##
## Created by: Qt User Interface Compiler version 5.15.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *

from src.mg_button_history import MgButtonHistory

# import multigit_resources_rc

class Ui_ApplyMgitFile(object):
    def setupUi(self, ApplyMgitFile: QDialog) -> None:
        if not ApplyMgitFile.objectName():
            ApplyMgitFile.setObjectName(u"ApplyMgitFile")
        ApplyMgitFile.resize(1010, 614)
        self.verticalLayout_4 = QVBoxLayout(ApplyMgitFile)
        self.verticalLayout_4.setObjectName(u"verticalLayout_4")
        self.horizontalLayout_3 = QHBoxLayout()
        self.horizontalLayout_3.setObjectName(u"horizontalLayout_3")
        self.label_2 = QLabel(ApplyMgitFile)
        self.label_2.setObjectName(u"label_2")
        sizePolicy = QSizePolicy(QSizePolicy.Preferred, QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(1)
        sizePolicy.setHeightForWidth(self.label_2.sizePolicy().hasHeightForWidth())
        self.label_2.setSizePolicy(sizePolicy)
        font = QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_2.setFont(font)

        self.horizontalLayout_3.addWidget(self.label_2)

        self.lineEditDestDir = QLineEdit(ApplyMgitFile)
        self.lineEditDestDir.setObjectName(u"lineEditDestDir")
        sizePolicy1 = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Preferred)
        sizePolicy1.setHorizontalStretch(10)
        sizePolicy1.setVerticalStretch(0)
        sizePolicy1.setHeightForWidth(self.lineEditDestDir.sizePolicy().hasHeightForWidth())
        self.lineEditDestDir.setSizePolicy(sizePolicy1)
        font1 = QFont()
        font1.setBold(False)
        font1.setWeight(50)
        self.lineEditDestDir.setFont(font1)
        self.lineEditDestDir.setReadOnly(True)

        self.horizontalLayout_3.addWidget(self.lineEditDestDir)


        self.verticalLayout_4.addLayout(self.horizontalLayout_3)

        self.groupBox = QGroupBox(ApplyMgitFile)
        self.groupBox.setObjectName(u"groupBox")
        sizePolicy2 = QSizePolicy(QSizePolicy.Preferred, QSizePolicy.Fixed)
        sizePolicy2.setHorizontalStretch(0)
        sizePolicy2.setVerticalStretch(1)
        sizePolicy2.setHeightForWidth(self.groupBox.sizePolicy().hasHeightForWidth())
        self.groupBox.setSizePolicy(sizePolicy2)
        self.groupBox.setFont(font)
        self.groupBox.setFlat(False)
        self.groupBox.setCheckable(False)
        self.gridLayout = QGridLayout(self.groupBox)
        self.gridLayout.setObjectName(u"gridLayout")
        self.label = QLabel(self.groupBox)
        self.label.setObjectName(u"label")
        self.label.setFont(font1)

        self.gridLayout.addWidget(self.label, 0, 0, 1, 4)

        self.pushButtonChooseMgitFile = QPushButton(self.groupBox)
        self.pushButtonChooseMgitFile.setObjectName(u"pushButtonChooseMgitFile")
        sizePolicy3 = QSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)
        sizePolicy3.setHorizontalStretch(1)
        sizePolicy3.setVerticalStretch(0)
        sizePolicy3.setHeightForWidth(self.pushButtonChooseMgitFile.sizePolicy().hasHeightForWidth())
        self.pushButtonChooseMgitFile.setSizePolicy(sizePolicy3)
        self.pushButtonChooseMgitFile.setMinimumSize(QSize(24, 0))
        self.pushButtonChooseMgitFile.setFont(font1)
        icon = QIcon()
        icon.addFile(u":/img/icons8-open-folder-64.png", QSize(), QIcon.Normal, QIcon.Off)
        self.pushButtonChooseMgitFile.setIcon(icon)

        self.gridLayout.addWidget(self.pushButtonChooseMgitFile, 1, 3, 1, 1)

        self.lineEditMgitFile = QLineEdit(self.groupBox)
        self.lineEditMgitFile.setObjectName(u"lineEditMgitFile")
        sizePolicy4 = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        sizePolicy4.setHorizontalStretch(10)
        sizePolicy4.setVerticalStretch(0)
        sizePolicy4.setHeightForWidth(self.lineEditMgitFile.sizePolicy().hasHeightForWidth())
        self.lineEditMgitFile.setSizePolicy(sizePolicy4)
        self.lineEditMgitFile.setFont(font1)

        self.gridLayout.addWidget(self.lineEditMgitFile, 1, 1, 1, 1)

        self.historyButtonMgitFile = MgButtonHistory(self.groupBox)
        self.historyButtonMgitFile.setObjectName(u"historyButtonMgitFile")
        icon1 = QIcon()
        icon1.addFile(u":/img/icons8-history-64.png", QSize(), QIcon.Normal, QIcon.Off)
        self.historyButtonMgitFile.setIcon(icon1)

        self.gridLayout.addWidget(self.historyButtonMgitFile, 1, 2, 1, 1)


        self.verticalLayout_4.addWidget(self.groupBox)

        self.groupBox_4 = QGroupBox(ApplyMgitFile)
        self.groupBox_4.setObjectName(u"groupBox_4")
        self.groupBox_4.setFont(font)
        self.verticalLayout_3 = QVBoxLayout(self.groupBox_4)
        self.verticalLayout_3.setObjectName(u"verticalLayout_3")
        self.label_3 = QLabel(self.groupBox_4)
        self.label_3.setObjectName(u"label_3")
        self.label_3.setFont(font1)

        self.verticalLayout_3.addWidget(self.label_3)

        self.horizontalLayout_4 = QHBoxLayout()
        self.horizontalLayout_4.setObjectName(u"horizontalLayout_4")
        self.lineEditUsername = QLineEdit(self.groupBox_4)
        self.lineEditUsername.setObjectName(u"lineEditUsername")
        sizePolicy5 = QSizePolicy(QSizePolicy.Preferred, QSizePolicy.Fixed)
        sizePolicy5.setHorizontalStretch(0)
        sizePolicy5.setVerticalStretch(0)
        sizePolicy5.setHeightForWidth(self.lineEditUsername.sizePolicy().hasHeightForWidth())
        self.lineEditUsername.setSizePolicy(sizePolicy5)

        self.horizontalLayout_4.addWidget(self.lineEditUsername)

        self.historyButtonUsername = MgButtonHistory(self.groupBox_4)
        self.historyButtonUsername.setObjectName(u"historyButtonUsername")
        sizePolicy6 = QSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)
        sizePolicy6.setHorizontalStretch(0)
        sizePolicy6.setVerticalStretch(0)
        sizePolicy6.setHeightForWidth(self.historyButtonUsername.sizePolicy().hasHeightForWidth())
        self.historyButtonUsername.setSizePolicy(sizePolicy6)
        self.historyButtonUsername.setIcon(icon1)

        self.horizontalLayout_4.addWidget(self.historyButtonUsername)


        self.verticalLayout_3.addLayout(self.horizontalLayout_4)


        self.verticalLayout_4.addWidget(self.groupBox_4)

        self.tabWidgetProject = QTabWidget(ApplyMgitFile)
        self.tabWidgetProject.setObjectName(u"tabWidgetProject")
        sizePolicy7 = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        sizePolicy7.setHorizontalStretch(0)
        sizePolicy7.setVerticalStretch(10)
        sizePolicy7.setHeightForWidth(self.tabWidgetProject.sizePolicy().hasHeightForWidth())
        self.tabWidgetProject.setSizePolicy(sizePolicy7)
        self.tabProject = QWidget()
        self.tabProject.setObjectName(u"tabProject")
        self.verticalLayout = QVBoxLayout(self.tabProject)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.horizontalLayout_2 = QHBoxLayout()
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.horizontalLayout_2.setSizeConstraint(QLayout.SetNoConstraint)
        self.labelProjDesc = QLabel(self.tabProject)
        self.labelProjDesc.setObjectName(u"labelProjDesc")
        sizePolicy.setHeightForWidth(self.labelProjDesc.sizePolicy().hasHeightForWidth())
        self.labelProjDesc.setSizePolicy(sizePolicy)
        self.labelProjDesc.setFont(font)

        self.horizontalLayout_2.addWidget(self.labelProjDesc)

        self.textEditProjectDesc = QTextEdit(self.tabProject)
        self.textEditProjectDesc.setObjectName(u"textEditProjectDesc")
        sizePolicy8 = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Preferred)
        sizePolicy8.setHorizontalStretch(0)
        sizePolicy8.setVerticalStretch(1)
        sizePolicy8.setHeightForWidth(self.textEditProjectDesc.sizePolicy().hasHeightForWidth())
        self.textEditProjectDesc.setSizePolicy(sizePolicy8)
        self.textEditProjectDesc.setMaximumSize(QSize(16777215, 50))
        self.textEditProjectDesc.setReadOnly(True)

        self.horizontalLayout_2.addWidget(self.textEditProjectDesc)

        self.horizontalLayout_2.setStretch(1, 1)

        self.verticalLayout.addLayout(self.horizontalLayout_2)

        self.treeWidgetRepoList = QTreeWidget(self.tabProject)
        self.treeWidgetRepoList.setObjectName(u"treeWidgetRepoList")
        sizePolicy9 = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        sizePolicy9.setHorizontalStretch(0)
        sizePolicy9.setVerticalStretch(20)
        sizePolicy9.setHeightForWidth(self.treeWidgetRepoList.sizePolicy().hasHeightForWidth())
        self.treeWidgetRepoList.setSizePolicy(sizePolicy9)
        self.treeWidgetRepoList.setRootIsDecorated(False)
        self.treeWidgetRepoList.setUniformRowHeights(True)
        self.treeWidgetRepoList.setItemsExpandable(False)
        self.treeWidgetRepoList.setSortingEnabled(True)
        self.treeWidgetRepoList.setColumnCount(3)

        self.verticalLayout.addWidget(self.treeWidgetRepoList)

        self.tabWidgetProject.addTab(self.tabProject, "")
        self.tabConfFile = QWidget()
        self.tabConfFile.setObjectName(u"tabConfFile")
        self.verticalLayout_2 = QVBoxLayout(self.tabConfFile)
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.textEditProj = QTextEdit(self.tabConfFile)
        self.textEditProj.setObjectName(u"textEditProj")

        self.verticalLayout_2.addWidget(self.textEditProj)

        self.tabWidgetProject.addTab(self.tabConfFile, "")

        self.verticalLayout_4.addWidget(self.tabWidgetProject)

        self.groupBox_3 = QGroupBox(ApplyMgitFile)
        self.groupBox_3.setObjectName(u"groupBox_3")
        self.horizontalLayout = QHBoxLayout(self.groupBox_3)
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.horizontalSpacer = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)

        self.horizontalLayout.addItem(self.horizontalSpacer)

        self.buttonBox = QDialogButtonBox(self.groupBox_3)
        self.buttonBox.setObjectName(u"buttonBox")
        self.buttonBox.setOrientation(Qt.Horizontal)
        self.buttonBox.setStandardButtons(QDialogButtonBox.Cancel)

        self.horizontalLayout.addWidget(self.buttonBox)


        self.verticalLayout_4.addWidget(self.groupBox_3)

        QWidget.setTabOrder(self.lineEditDestDir, self.lineEditMgitFile)
        QWidget.setTabOrder(self.lineEditMgitFile, self.historyButtonMgitFile)
        QWidget.setTabOrder(self.historyButtonMgitFile, self.pushButtonChooseMgitFile)
        QWidget.setTabOrder(self.pushButtonChooseMgitFile, self.lineEditUsername)
        QWidget.setTabOrder(self.lineEditUsername, self.historyButtonUsername)
        QWidget.setTabOrder(self.historyButtonUsername, self.tabWidgetProject)
        QWidget.setTabOrder(self.tabWidgetProject, self.textEditProjectDesc)
        QWidget.setTabOrder(self.textEditProjectDesc, self.treeWidgetRepoList)
        QWidget.setTabOrder(self.treeWidgetRepoList, self.textEditProj)

        self.retranslateUi(ApplyMgitFile)
        self.buttonBox.accepted.connect(ApplyMgitFile.accept)
        self.buttonBox.rejected.connect(ApplyMgitFile.reject)

        self.tabWidgetProject.setCurrentIndex(0)


        QMetaObject.connectSlotsByName(ApplyMgitFile)
    # setupUi

    def retranslateUi(self, ApplyMgitFile: QDialog) -> None:
        ApplyMgitFile.setWindowTitle(QCoreApplication.translate("ApplyMgitFile", u"Apply Multigit File", None))
        self.label_2.setText(QCoreApplication.translate("ApplyMgitFile", u"Base directory :", None))
        self.groupBox.setTitle(QCoreApplication.translate("ApplyMgitFile", u"Multigit file", None))
        self.label.setText(QCoreApplication.translate("ApplyMgitFile", u"Select a multigit file (extension .mgit)", None))
#if QT_CONFIG(tooltip)
        self.pushButtonChooseMgitFile.setToolTip(QCoreApplication.translate("ApplyMgitFile", u"Choose multigit file", None))
#endif // QT_CONFIG(tooltip)
        self.pushButtonChooseMgitFile.setText("")
#if QT_CONFIG(tooltip)
        self.historyButtonMgitFile.setToolTip(QCoreApplication.translate("ApplyMgitFile", u"Select recent multigit file", None))
#endif // QT_CONFIG(tooltip)
        self.historyButtonMgitFile.setText("")
        self.groupBox_4.setTitle(QCoreApplication.translate("ApplyMgitFile", u" User Name", None))
        self.label_3.setText(QCoreApplication.translate("ApplyMgitFile", u"User name in the git URL when cloning", None))
#if QT_CONFIG(tooltip)
        self.historyButtonUsername.setToolTip(QCoreApplication.translate("ApplyMgitFile", u"Select recent username", None))
#endif // QT_CONFIG(tooltip)
        self.historyButtonUsername.setText("")
        self.labelProjDesc.setText(QCoreApplication.translate("ApplyMgitFile", u"Project description :", None))
        self.textEditProjectDesc.setHtml(QCoreApplication.translate("ApplyMgitFile", u"<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'MS Shell Dlg 2'; font-size:9.75pt; font-weight:400; font-style:normal;\">\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-size:8.25pt;\"><br /></p></body></html>", None))
        ___qtreewidgetitem = self.treeWidgetRepoList.headerItem()
        ___qtreewidgetitem.setText(2, QCoreApplication.translate("ApplyMgitFile", u"Path", None));
        ___qtreewidgetitem.setText(1, QCoreApplication.translate("ApplyMgitFile", u"HEAD target", None));
        ___qtreewidgetitem.setText(0, QCoreApplication.translate("ApplyMgitFile", u"Repository", None));
        self.tabWidgetProject.setTabText(self.tabWidgetProject.indexOf(self.tabProject), QCoreApplication.translate("ApplyMgitFile", u"Repositories", None))
        self.tabWidgetProject.setTabText(self.tabWidgetProject.indexOf(self.tabConfFile), QCoreApplication.translate("ApplyMgitFile", u"Multigit file content", None))
        self.groupBox_3.setTitle("")
    # retranslateUi




# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'ui_export_mgit.ui'
##
## Created by: Qt User Interface Compiler version 5.15.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *

from src.mg_button_history import MgButtonHistory

# import multigit_resources_rc

class Ui_ExportMgit(object):
    def setupUi(self, ExportMgit: QDialog) -> None:
        if not ExportMgit.objectName():
            ExportMgit.setObjectName(u"ExportMgit")
        ExportMgit.resize(646, 196)
        self.verticalLayout = QVBoxLayout(ExportMgit)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.label = QLabel(ExportMgit)
        self.label.setObjectName(u"label")
        font = QFont()
        font.setBold(False)
        font.setWeight(50)
        self.label.setFont(font)

        self.verticalLayout.addWidget(self.label)

        self.horizontalLayout_2 = QHBoxLayout()
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.lineEditMgitFile = QLineEdit(ExportMgit)
        self.lineEditMgitFile.setObjectName(u"lineEditMgitFile")
        sizePolicy = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(10)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEditMgitFile.sizePolicy().hasHeightForWidth())
        self.lineEditMgitFile.setSizePolicy(sizePolicy)
        self.lineEditMgitFile.setFont(font)

        self.horizontalLayout_2.addWidget(self.lineEditMgitFile)

        self.historyButtonMgitFile = MgButtonHistory(ExportMgit)
        self.historyButtonMgitFile.setObjectName(u"historyButtonMgitFile")
        icon = QIcon()
        icon.addFile(u":/img/icons8-history-64.png", QSize(), QIcon.Normal, QIcon.Off)
        self.historyButtonMgitFile.setIcon(icon)

        self.horizontalLayout_2.addWidget(self.historyButtonMgitFile)

        self.pushButtonChooseMgitFile = QPushButton(ExportMgit)
        self.pushButtonChooseMgitFile.setObjectName(u"pushButtonChooseMgitFile")
        sizePolicy1 = QSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)
        sizePolicy1.setHorizontalStretch(1)
        sizePolicy1.setVerticalStretch(0)
        sizePolicy1.setHeightForWidth(self.pushButtonChooseMgitFile.sizePolicy().hasHeightForWidth())
        self.pushButtonChooseMgitFile.setSizePolicy(sizePolicy1)
        self.pushButtonChooseMgitFile.setMinimumSize(QSize(24, 0))
        self.pushButtonChooseMgitFile.setFont(font)
        icon1 = QIcon()
        icon1.addFile(u":/img/icons8-open-folder-64.png", QSize(), QIcon.Normal, QIcon.Off)
        self.pushButtonChooseMgitFile.setIcon(icon1)

        self.horizontalLayout_2.addWidget(self.pushButtonChooseMgitFile)


        self.verticalLayout.addLayout(self.horizontalLayout_2)

        self.label_2 = QLabel(ExportMgit)
        self.label_2.setObjectName(u"label_2")

        self.verticalLayout.addWidget(self.label_2)

        self.lineEditDescMgit = QLineEdit(ExportMgit)
        self.lineEditDescMgit.setObjectName(u"lineEditDescMgit")

        self.verticalLayout.addWidget(self.lineEditDescMgit)

        self.verticalSpacer = QSpacerItem(20, 48, QSizePolicy.Minimum, QSizePolicy.Expanding)

        self.verticalLayout.addItem(self.verticalSpacer)

        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.pushButtonCancel = QPushButton(ExportMgit)
        self.pushButtonCancel.setObjectName(u"pushButtonCancel")

        self.horizontalLayout.addWidget(self.pushButtonCancel)

        self.pushButtonExportSnapshot = QPushButton(ExportMgit)
        self.pushButtonExportSnapshot.setObjectName(u"pushButtonExportSnapshot")

        self.horizontalLayout.addWidget(self.pushButtonExportSnapshot)

        self.pushButtonExportProject = QPushButton(ExportMgit)
        self.pushButtonExportProject.setObjectName(u"pushButtonExportProject")

        self.horizontalLayout.addWidget(self.pushButtonExportProject)


        self.verticalLayout.addLayout(self.horizontalLayout)

        QWidget.setTabOrder(self.lineEditMgitFile, self.historyButtonMgitFile)
        QWidget.setTabOrder(self.historyButtonMgitFile, self.pushButtonChooseMgitFile)
        QWidget.setTabOrder(self.pushButtonChooseMgitFile, self.lineEditDescMgit)
        QWidget.setTabOrder(self.lineEditDescMgit, self.pushButtonCancel)
        QWidget.setTabOrder(self.pushButtonCancel, self.pushButtonExportSnapshot)
        QWidget.setTabOrder(self.pushButtonExportSnapshot, self.pushButtonExportProject)

        self.retranslateUi(ExportMgit)

        QMetaObject.connectSlotsByName(ExportMgit)
    # setupUi

    def retranslateUi(self, ExportMgit: QDialog) -> None:
        ExportMgit.setWindowTitle(QCoreApplication.translate("ExportMgit", u"Export to multigit file", None))
        self.label.setText(QCoreApplication.translate("ExportMgit", u"Select destination multigit file :", None))
#if QT_CONFIG(tooltip)
        self.historyButtonMgitFile.setToolTip(QCoreApplication.translate("ExportMgit", u"Choose recent multigit file", None))
#endif // QT_CONFIG(tooltip)
        self.historyButtonMgitFile.setText("")
#if QT_CONFIG(tooltip)
        self.pushButtonChooseMgitFile.setToolTip(QCoreApplication.translate("ExportMgit", u"Select multigit file", None))
#endif // QT_CONFIG(tooltip)
        self.pushButtonChooseMgitFile.setText("")
        self.label_2.setText(QCoreApplication.translate("ExportMgit", u"Description of your multigit file", None))
#if QT_CONFIG(tooltip)
        self.lineEditDescMgit.setToolTip(QCoreApplication.translate("ExportMgit", u"Description of your multigit file", None))
#endif // QT_CONFIG(tooltip)
        self.pushButtonCancel.setText(QCoreApplication.translate("ExportMgit", u"Cancel", None))
#if QT_CONFIG(tooltip)
        self.pushButtonExportSnapshot.setToolTip(QCoreApplication.translate("ExportMgit", u"Export all repos with detached HEAD set on last commit, to create a snapshot of the project", None))
#endif // QT_CONFIG(tooltip)
        self.pushButtonExportSnapshot.setText(QCoreApplication.translate("ExportMgit", u"Export as snapshot (commit)", None))
#if QT_CONFIG(tooltip)
        self.pushButtonExportProject.setToolTip(QCoreApplication.translate("ExportMgit", u"Export all repos with HEAD set on current branch or tag for regular project development", None))
#endif // QT_CONFIG(tooltip)
        self.pushButtonExportProject.setText(QCoreApplication.translate("ExportMgit", u"Export as project (tag/branch)", None))
    # retranslateUi




# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'ui_export_csv.ui'
##
## Created by: Qt User Interface Compiler version 5.15.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *


class Ui_dialogCsvExport(object):
    def setupUi(self, dialogCsvExport: QDialog) -> None:
        if not dialogCsvExport.objectName():
            dialogCsvExport.setObjectName(u"dialogCsvExport")
        dialogCsvExport.resize(438, 324)
        self.verticalLayout = QVBoxLayout(dialogCsvExport)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.gridLayout = QGridLayout()
        self.gridLayout.setObjectName(u"gridLayout")
        self.label = QLabel(dialogCsvExport)
        self.label.setObjectName(u"label")

        self.gridLayout.addWidget(self.label, 0, 0, 1, 2)

        self.checkPath = QCheckBox(dialogCsvExport)
        self.checkPath.setObjectName(u"checkPath")
        self.checkPath.setChecked(True)

        self.gridLayout.addWidget(self.checkPath, 1, 0, 1, 1)

        self.checkCommitSha1 = QCheckBox(dialogCsvExport)
        self.checkCommitSha1.setObjectName(u"checkCommitSha1")

        self.gridLayout.addWidget(self.checkCommitSha1, 1, 1, 1, 1)

        self.checkHead = QCheckBox(dialogCsvExport)
        self.checkHead.setObjectName(u"checkHead")
        self.checkHead.setChecked(True)

        self.gridLayout.addWidget(self.checkHead, 2, 0, 1, 1)

        self.checkCommitDate = QCheckBox(dialogCsvExport)
        self.checkCommitDate.setObjectName(u"checkCommitDate")

        self.gridLayout.addWidget(self.checkCommitDate, 2, 1, 1, 1)

        self.checkCurrentBranch = QCheckBox(dialogCsvExport)
        self.checkCurrentBranch.setObjectName(u"checkCurrentBranch")
        self.checkCurrentBranch.setChecked(True)

        self.gridLayout.addWidget(self.checkCurrentBranch, 3, 0, 1, 1)

        self.checkUrl = QCheckBox(dialogCsvExport)
        self.checkUrl.setObjectName(u"checkUrl")
        self.checkUrl.setChecked(True)

        self.gridLayout.addWidget(self.checkUrl, 3, 1, 1, 1)

        self.checkCurrentTag = QCheckBox(dialogCsvExport)
        self.checkCurrentTag.setObjectName(u"checkCurrentTag")
        self.checkCurrentTag.setChecked(True)

        self.gridLayout.addWidget(self.checkCurrentTag, 4, 0, 1, 1)


        self.verticalLayout.addLayout(self.gridLayout)

        self.verticalSpacer = QSpacerItem(20, 34, QSizePolicy.Minimum, QSizePolicy.Expanding)

        self.verticalLayout.addItem(self.verticalSpacer)

        self.buttonBox = QDialogButtonBox(dialogCsvExport)
        self.buttonBox.setObjectName(u"buttonBox")
        self.buttonBox.setOrientation(Qt.Horizontal)
        self.buttonBox.setStandardButtons(QDialogButtonBox.Cancel|QDialogButtonBox.Ok)

        self.verticalLayout.addWidget(self.buttonBox)

        QWidget.setTabOrder(self.checkPath, self.checkHead)
        QWidget.setTabOrder(self.checkHead, self.checkCurrentBranch)
        QWidget.setTabOrder(self.checkCurrentBranch, self.checkCurrentTag)
        QWidget.setTabOrder(self.checkCurrentTag, self.checkCommitSha1)
        QWidget.setTabOrder(self.checkCommitSha1, self.checkCommitDate)
        QWidget.setTabOrder(self.checkCommitDate, self.checkUrl)

        self.retranslateUi(dialogCsvExport)
        self.buttonBox.accepted.connect(dialogCsvExport.accept)
        self.buttonBox.rejected.connect(dialogCsvExport.reject)

        QMetaObject.connectSlotsByName(dialogCsvExport)
    # setupUi

    def retranslateUi(self, dialogCsvExport: QDialog) -> None:
        dialogCsvExport.setWindowTitle(QCoreApplication.translate("dialogCsvExport", u"Export to CSV", None))
        self.label.setText(QCoreApplication.translate("dialogCsvExport", u"Choose which fields to export :\n"
"", None))
        self.checkPath.setText(QCoreApplication.translate("dialogCsvExport", u"Path", None))
        self.checkCommitSha1.setText(QCoreApplication.translate("dialogCsvExport", u"commit sha1", None))
        self.checkHead.setText(QCoreApplication.translate("dialogCsvExport", u"HEAD", None))
        self.checkCommitDate.setText(QCoreApplication.translate("dialogCsvExport", u"commit date", None))
        self.checkCurrentBranch.setText(QCoreApplication.translate("dialogCsvExport", u"current branch", None))
        self.checkUrl.setText(QCoreApplication.translate("dialogCsvExport", u"url", None))
        self.checkCurrentTag.setText(QCoreApplication.translate("dialogCsvExport", u"current tag", None))
    # retranslateUi




#    Copyright (c) 2019-2023 IDEMIA
#    Author: IDEMIA (Philippe Fremy, Florent Oulieres)
# 
#     Licensed under the Apache License, Version 2.0 (the "License");
#     you may not use this file except in compliance with the License.
#     You may obtain a copy of the License at
# 
#         http://www.apache.org/licenses/LICENSE-2.0
# 
#     Unless required by applicable law or agreed to in writing, software
#     distributed under the License is distributed on an "AS IS" BASIS,
#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#     See the License for the specific language governing permissions and
#     limitations under the License.
#


from typing import Union, Any
import unittest, tempfile, datetime, collections, pathlib, os, stat, shutil
from logging import warning

import os
import os.path

from multigit import init_logging
import src.mg_tools
from src.mg_tools import ExecGit
from src.mg_const import MSG_EMPTY_REPO, MSG_NO_COMMIT, MSG_LOCAL_BRANCH, MSG_REMOTE_SYNCHRO_OK, SHORT_SHA1_NB_DIGITS
from src.mg_repo_info import MgRepoInfo, MultiRepo

# More tests to write
# * detached head from a remote branch
#   + deleting the remote branch
# * clone remote branch + delete the remote branch

AUTHOR_NAME = 'philou'
AUTHOR_EMAIL = 'phil_tests@github'

LONG_TESTS = True

init_logging(run_from_tests=True, debug_activated=False) # set to True, True when you want debug on stdout

RepoInfoTuple = collections.namedtuple('RepoInfoTuple',
                   ["name", "head", "branch", "tag", "remote_branch", "status", "fullpath",
                    "last_commit", "remote_synchro", "tags", "branches_local", "branches_remote"],
                       defaults=["", "", "", "", "", "", "", "", "", None, (), ()])


def git_exec(*args: str, gitdir: Union[str, pathlib.Path] = '', allow_errors: bool = False) -> str:
    '''Run git with the arguments passed in *args and return the stdout of the command.
    Raises an exception if the command did not return with 0 status.
    '''
    if gitdir:
        args = tuple([ '-C', str(gitdir) ] + list(args))

    return ExecGit.exec_blocking(args, '', allow_errors)


GIT_SUPPORTS_SETTING_BRANCH_NAME = False
DEFAULT_BRANCH_NAME='master'

GIT_VERSION = git_exec('--version').split(' ')[2]
if GIT_VERSION >= '2.28':
    GIT_SUPPORTS_SETTING_BRANCH_NAME = True
    DEFAULT_BRANCH_NAME='main'


def rmtree_failsafe(p: Union[str, pathlib.Path]) -> None:
    # print('rmtree_failsafe(%s)' % p)
    try:
        dirlist = list(os.scandir(p))
    except PermissionError:
        # nothing we can do with this directory
        return

    local_error = []

    def rmtree_onerror(_function: Any, path: Any, _excinfo: Any) -> None:
        try:
            # deletion errors are usually because a file is readonly
            os.chmod(path, stat.S_IWRITE)
            os.unlink(path)
        except Exception as exc:
            # exception again !
            local_error.append('''On path %s
    %s''' % (str(path), str(exc)))
            warning(local_error[-1])

    shutil.rmtree(p, onerror=rmtree_onerror)

    if len(local_error) > 2:
        local_error = local_error[:2] + ['And more errors...\n']

    return '\n'.join(local_error)


def to_named_tuple(ric: MgRepoInfo) -> RepoInfoTuple:
        return RepoInfoTuple(
            name=ric.name,
            fullpath=ric.fullpath,
            branch=ric.branch,
            head=ric.head,
            tag=ric.tag,
            remote_branch=ric.remote_branch,
            status=ric.status,
            last_commit=ric.last_commit,
            remote_synchro=ric.remote_synchro,
            tags=ric.tags,
            branches_remote=tuple(ric.branches_remote),
            branches_local=tuple(ric.branches_local),
        )


def git_init_repo(repo: str) -> None:
    '''Init the repository and creates commit config'''
    if GIT_SUPPORTS_SETTING_BRANCH_NAME:
        branch_args = ['-b', 'main']
    else:
        branch_args = []
    git_exec(*(['init'] + branch_args + [repo]))
    git_set_commit_author(repo)


def git_set_commit_author(repo: str) -> None:
    git_exec('config', 'user.email', AUTHOR_EMAIL , gitdir=repo)
    git_exec('config', 'user.name', AUTHOR_NAME , gitdir=repo)


def git_commit(repo: str, msg: str, allow_errors: bool = False) -> str:
    '''Commit the repo with the message passed in argument and return the SHA1 of the commit.'''
    out = git_exec('-C', repo, 'commit', '-m', msg, allow_errors=allow_errors)
    sha1 = out.split(']')[0].split(' ')[-1]
    return sha1


def add_content(repo: Union[str, pathlib.Path], fname: str, content: str = '') -> str:
    '''create or append to a file and commit it. Return the sha1 of the commit'''
    olddir = os.getcwd()
    os.chdir(repo)
    if not content:
        content = generate_content()
    if not content.endswith('\n'):
        content += '\n'

    with open(fname,'a') as f:
        f.write(content)
    git_exec('add', fname)
    sha1 = git_commit('.', '"extend %s"' % fname)
    os.chdir(olddir)
    return sha1


content_idx = 0
def generate_content():
    '''Return the text "line X" increasing on each call'''
    global content_idx
    content_idx += 1
    return 'line %d\n' % content_idx


#######################################################
#           Tests for RepoInfo
#######################################################

app = None
old_FORCE_ASYNC_TO_BLOCKING_CALLS = False
def setUpModule() -> None:
    '''Make all git calls blocking during the duration of the test'''
    global old_FORCE_ASYNC_TO_BLOCKING_CALLS
    old_FORCE_ASYNC_TO_BLOCKING_CALLS = src.mg_tools.FORCE_ASYNC_TO_BLOCKING_CALLS
    src.mg_tools.FORCE_ASYNC_TO_BLOCKING_CALLS = True


def tearDownModule() -> None:
    '''Restore previous state after all tests execution'''
    src.mg_tools.FORCE_ASYNC_TO_BLOCKING_CALLS = old_FORCE_ASYNC_TO_BLOCKING_CALLS


# noinspection PyArgumentList,PyBroadException
class TestRepoInfo(unittest.TestCase):
    '''This test creates git repositories in TEMP/test_multigit/YYYYMMDD_HHMMSS

    The tests are serialized. We tried to make them independant but it was too messy
    and too long to recreate independant states.

    A repository is put into various states (empty, one commit, modified, ...) and
    a test function is run for each of these tests:
    * run_test_on_empty_git()
    * run_test_on_main()
    * run_test_on_main_modified_files()
    * run_test_on_main_with_one_tag()
    * run_test_on_tag()
    * run_test_on_tag2_with_tag3(self.dir1, 'tag2', 'tag3')
    * run_test_on_commit()
    * run_test_on_tag_like_sha1()
    * run_test_on_other_branch()
    * run_test_on_other_branch_merged_main()


    if LONG_TEST is set to True, an additional set of longer tests is run, involving clones:
    * run_clone_test_on_empty_git(self.dir1, self.dir3)
    * run_clone_test_on_origin_main_2_commits()
    * run_clone_test_on_origin_main_more_commits(self.dir1, self.dir3)

    '''

    @classmethod
    def setUpClass(cls: 'TestRepoInfo') -> None:
        cls.tempdir_setUp()
        cls.init_dir = os.getcwd()

    @classmethod
    def tearDownClass(cls: 'TestRepoInfo') -> None:
        cls.tempdir_tearDown()
        os.chdir(cls.init_dir)

    @classmethod
    def tempdir_setUp(cls: 'TestRepoInfo') -> None:
        cls.tempdir = pathlib.Path(tempfile.gettempdir()) / 'test_multigit'
        cls.tempdir.mkdir(exist_ok=True)
        cls.gitdir = cls.tempdir / datetime.datetime.now().isoformat().replace(':', '_').split('.')[0]
        cls.gitdir.mkdir()
        print('Creating test git directory: %s' % cls.gitdir)


    @classmethod
    def tempdir_tearDown(cls: 'TestRepoInfo') -> None:
        if not os.path.exists(cls.tempdir):
            return
        # try to remove our own stuff
        rmtree_failsafe(cls.gitdir)

        # try to remove leftofers from previous tests as well
        rmtree_failsafe(cls.tempdir)

    def test_repo_info_fill_branch(self) -> None:
        ri = MgRepoInfo('toto', '.', '.')
        self.assertEqual(ri.branches_remote, [])
        self.assertEqual(ri.branches_local, [])
        self.assertEqual(ri.branches_filled, False)

        ri.cb_fill_branches_done('toto', 0, f'''
  dev
  feat/f-1
* feat/f-2
  {DEFAULT_BRANCH_NAME}
  remotes/origin/HEAD -> origin/fix/toto
  remotes/origin/b2
  remotes/origin/dev
  remotes/origin/feat/Clone_1
  remotes/origin/feat/GF_2
        
''')
        self.assertEqual(ri.branches_local, ['dev', 'feat/f-1', 'feat/f-2', DEFAULT_BRANCH_NAME])
        self.assertEqual(ri.branches_remote, [
            'origin/b2',
            'origin/dev',
            'origin/feat/Clone_1',
            'origin/feat/GF_2',
            ])
        self.assertEqual(ri.branches_filled, True)

        # when head is detached, we have an extra line of information
        ri = MgRepoInfo('toto', '.', '.')
        ri.cb_fill_branches_done('toto', 0, f''' * (HEAD detached at tag1)
  {DEFAULT_BRANCH_NAME}
''')
        self.assertEqual(ri.branches_local, [DEFAULT_BRANCH_NAME])
        self.assertEqual(ri.branches_remote, [])



    def test_repo_info(self) -> None:
        self.dir1 = pathlib.Path(self.gitdir) / 'dir1'
        self.dir1.mkdir()
        os.chdir(self.dir1)
        git_init_repo('.')

        self.run_test_on_empty_git(self.dir1)

        if LONG_TESTS:
            self.dir3 = pathlib.Path(self.gitdir) / 'dir3'
            self.dir3.mkdir()
            self.run_clone_test_on_empty_git(self.dir1, self.dir3)

        # add one file, one commit, no tag
        sha1 = add_content(self.dir1, 'file1')
        self.run_test_on_main(self.dir1, 'file1', sha1)

        # add 2nd commit
        # checkout is still on main
        sha1 = add_content(self.dir1, 'file2')
        self.run_test_on_main(self.dir1, 'file2', sha1)

        if LONG_TESTS:
            self.run_clone_test_on_origin_main_2_commits(self.dir1, self.dir3)

        self.run_test_on_main_modified_files(self.dir1, sha1)

        # tag the file
        git_exec('tag', 'tag1')
        self.run_test_on_main_with_one_tag(self.dir1, 'tag1')

        # checkout tag1
        git_exec('checkout', 'tag1')
        self.run_test_on_tag(self.dir1, 'tag1')

        if LONG_TESTS:
            # checkout main, add 3rd commit, checkout tag1
            git_exec('checkout', DEFAULT_BRANCH_NAME)
            _sha1 = add_content(self.dir1, 'tata')

            self.run_clone_test_on_origin_main_more_commits(self.dir1, self.dir3)

            # clone a directory while it is pointing to a tag
            git_exec('checkout', 'tag1')
            ric = MgRepoInfo(str(self.dir1), '')
            ric.fill_last_commit()
            tag_sha1 = ric.commit_sha1

            self.dir4 = pathlib.Path(self.gitdir) / 'dir4'
            self.dir4.mkdir()

            self.run_clone_test_on_tag(self.dir1, 'tag1', self.dir4, tag_sha1)

        # checkout main, tag the file twice, checkout one tag
        git_exec('checkout', DEFAULT_BRANCH_NAME)
        _sha1 = add_content(self.dir1, 'tata')
        git_exec('tag', 'tag2')
        git_exec('tag', 'tag3')
        git_exec('checkout', 'tag2')
        self.run_test_on_tag2_with_tag3(self.dir1, 'tag2', 'tag3')

        # checkout main, add 4th commit, add 5th commit
        # checkout 4th commit
        git_exec('checkout', DEFAULT_BRANCH_NAME)
        sha1_4 = add_content(self.dir1, 'titi')
        sha1_5 = add_content(self.dir1, 'titi')
        git_exec('checkout', sha1_5)
        self.run_test_on_commit(self.dir1, sha1_5)

        if LONG_TESTS:
            # add tag looking like a sha1
            tag_like_sha1 = '112233AABB'
            git_exec('tag', tag_like_sha1)
            git_exec('checkout', sha1_4) # we change the current commit to fetch the information about tags
            git_exec('checkout', tag_like_sha1)
            self.run_test_on_tag_like_sha1(self.dir1, tag_like_sha1)

        # create branch branch1, add 2 commits
        git_exec('checkout', '-b', 'branch1')
        add_content(self.dir1, 'toto')
        sha1 = add_content(self.dir1, 'toto')
        self.run_test_on_other_branch(self.dir1, 'branch1', sha1)

        # merge main into branch1
        git_exec('merge', DEFAULT_BRANCH_NAME)
        self.run_test_on_other_branch_merged_main(self.dir1, 'branch1')

        # create branch2, fill it with content, merge it to main and delete it
        git_exec('checkout', '-b', 'branch2')
        add_content(self.dir1, 'toto')
        git_exec('checkout', DEFAULT_BRANCH_NAME)
        git_exec('merge', 'branch2')
        git_exec('branch', '-d', 'branch2')
        self.run_test_on_main_after_branch2_deleted(self.dir1, DEFAULT_BRANCH_NAME)


    def run_test_on_empty_git(self, dir1):
        print('    - run_test_on_empty_git')
        n, p, fp = str(dir1), dir1, str(dir1.resolve())

        # check refesh() effect
        ric = MgRepoInfo(n, fp)
        self.assertEqual(to_named_tuple(ric.refresh()),
             RepoInfoTuple(name=n,
                           head=MSG_EMPTY_REPO,
                           branch=MSG_EMPTY_REPO,
                           fullpath=fp,
                           status='OK',
                           last_commit=MSG_NO_COMMIT,
                           tags='',
                       ))
        self.assertEqual(ric.commit_sha1, '')

        # check ensure_last_commit() effect
        self.assertEqual(ric.last_commit, MSG_NO_COMMIT)
        ric.ensure_last_commit()
        self.assertEqual(ric.last_commit, MSG_NO_COMMIT)

        self.assertEqual(ric.branches_filled, False)
        ric.ensure_branches_filled()
        self.assertEqual(ric.branches_filled, True)
        self.assertEqual(ric.branches_remote, [])
        self.assertEqual(ric.branches_local, [])


        # check ensure_all_filled() does not fail
        ric = MgRepoInfo(n, fp)
        ric.ensure_all_filled()

        # check how modified files are reported
        with open(p/'tutu','a') as f:
            f.write('blurb' + '\n')

        self.assertEqual(to_named_tuple(ric.refresh()),
             RepoInfoTuple(name=n,
                           head=MSG_EMPTY_REPO,
                           branch=MSG_EMPTY_REPO,
                           fullpath=fp,
                           status='OK',
                           last_commit=MSG_NO_COMMIT,
                           tags=''
                        ))
        self.assertEqual(ric.commit_sha1, '')

        git_exec('add', 'tutu', gitdir=p)
        self.assertEqual(to_named_tuple(ric.refresh()),
                         RepoInfoTuple(name=n,
                                       head=MSG_EMPTY_REPO,
                                       branch=MSG_EMPTY_REPO,
                                       fullpath=fp,
                                       status='1 modified file',
                                       last_commit=MSG_NO_COMMIT,
                                       tags=''
                                       ))
        self.assertEqual(ric.commit_sha1, '')

        git_exec('rm', '-f', 'tutu', gitdir=p)
        self.assertEqual(to_named_tuple(ric.refresh()),
                         RepoInfoTuple(name=n,
                                       head=MSG_EMPTY_REPO,
                                       branch=MSG_EMPTY_REPO,
                                       fullpath=fp,
                                       status='OK',
                                       last_commit=MSG_NO_COMMIT,
                                       tags=''
                                       ))
        self.assertEqual(ric.commit_sha1, '')
        self.assertEqual((p/'tutu').exists(), False)
        return


    def run_test_on_main(self, dir1, fname, commit_sha1: str):
        print('    - run_test_on_main')
        n, p, fp = str(dir1), dir1, str(dir1.resolve())

        # check refesh() effect
        ric = MgRepoInfo(n, fp)
        self.assertEqual(to_named_tuple(ric.refresh()),
             RepoInfoTuple(name=n,
                           head=f'branch {DEFAULT_BRANCH_NAME}',
                           branch=DEFAULT_BRANCH_NAME,
                           remote_synchro = MSG_LOCAL_BRANCH,
                           fullpath=fp,
                           status='OK',
                           last_commit='',
                           tags=None,
                    ))
        self.assertEqual(ric.commit_sha1, None)
        ric.ensure_last_commit()
        self.assertTrue(ric.last_commit.startswith('commit %s' % commit_sha1))
        self.assertTrue(('extend %s' % fname) in ric.last_commit)
        self.assertTrue('Author:' in ric.last_commit)
        self.assertTrue('Date:' in ric.last_commit)

        ric.ensure_branches_filled()
        self.assertEqual(ric.branches_filled, True)
        self.assertEqual(ric.branches_remote, [])
        self.assertEqual(ric.branches_local, [DEFAULT_BRANCH_NAME])


    def run_test_on_main_modified_files(self, dir1, _sha1: str):
        print('    - run_test_on_main_modified_files')
        n, p, fp = str(dir1), dir1, str(dir1.resolve())

        ric = MgRepoInfo(n, fp)
        self.assertEqual(to_named_tuple(ric.refresh()),
                         RepoInfoTuple(name=n, head=f'branch {DEFAULT_BRANCH_NAME}', branch=DEFAULT_BRANCH_NAME,
                                       remote_synchro = MSG_LOCAL_BRANCH, fullpath=fp,
                                       status='OK',
                                       ))

        with open(p / 'file1', 'a') as f:
            f.write(generate_content())

        self.assertEqual(to_named_tuple(ric.refresh()),
                         RepoInfoTuple(name=n, head=f'branch {DEFAULT_BRANCH_NAME}', branch=DEFAULT_BRANCH_NAME,
                                       remote_synchro=MSG_LOCAL_BRANCH, fullpath=fp,
                                       status='1 modified file'
                                       ))

        with open(p / 'file3', 'a') as f:
            f.write(generate_content())
        git_exec('add', 'file3', gitdir=p)

        self.assertEqual(to_named_tuple(ric.refresh()),
                         RepoInfoTuple(name=n, head=f'branch {DEFAULT_BRANCH_NAME}', branch=DEFAULT_BRANCH_NAME,
                                       remote_synchro=MSG_LOCAL_BRANCH, fullpath=fp,
                                       status='2 modified files'
                                       ))

        (p/'file2').unlink()
        self.assertEqual(to_named_tuple(ric.refresh()),
                         RepoInfoTuple(name=n, head=f'branch {DEFAULT_BRANCH_NAME}', branch=DEFAULT_BRANCH_NAME,
                                       remote_synchro=MSG_LOCAL_BRANCH, fullpath=fp,
                                       status='3 modified files'
                                       ))

        git_exec('add', 'file1', gitdir=p)
        git_exec('rm', 'file2', gitdir=p)
        self.assertEqual(to_named_tuple(ric.refresh()),
                         RepoInfoTuple(name=n, head=f'branch {DEFAULT_BRANCH_NAME}', branch=DEFAULT_BRANCH_NAME,
                                       remote_synchro=MSG_LOCAL_BRANCH, fullpath=fp,
                                       status='3 modified files'
                                       ))

        git_commit(fp, 'check on modified files done')

        self.assertEqual(to_named_tuple(ric.refresh()),
                         RepoInfoTuple(name=n, head=f'branch {DEFAULT_BRANCH_NAME}', branch=DEFAULT_BRANCH_NAME,
                                       remote_synchro = MSG_LOCAL_BRANCH, fullpath=fp,
                                       status='OK',
                                       ))

    def run_test_on_main_with_one_tag(self, dir1, tag1: str):
        print('    - run_test_on_main_with_one_tag')
        n, p, fp = str(dir1), dir1, str(dir1.resolve())
        # check refesh() effect
        ric = MgRepoInfo(n, fp)
        self.assertEqual(to_named_tuple(ric.refresh()),
                         RepoInfoTuple(name=n,
                                       head=f'branch {DEFAULT_BRANCH_NAME}',
                                       branch=DEFAULT_BRANCH_NAME,
                                       remote_synchro = MSG_LOCAL_BRANCH,
                                       fullpath=fp,
                                       status='OK',
                                       last_commit='',
                                       tags=None,
                                       ))
        ric.ensure_tags()
        self.assertEqual(ric.tags, tag1)
        ric.ensure_all_filled() # check that no exception is raised

        ric.ensure_branches_filled()
        self.assertEqual(ric.branches_filled, True)
        self.assertEqual(ric.branches_remote, [])
        self.assertEqual(ric.branches_local, [DEFAULT_BRANCH_NAME])



    def run_test_on_tag(self, dir1, tag1: str):
        print('    - run_test_on_tag')
        n, p, fp = str(dir1), dir1, str(dir1.resolve())
        # check refesh() effect
        ric = MgRepoInfo(n, fp)
        ric.refresh()
        self.assertEqual(to_named_tuple(ric),
                         RepoInfoTuple(name=n,
                                       head='tag %s' % tag1,
                                       tag=tag1,
                                       branch='',
                                       remote_synchro='',
                                       fullpath=fp,
                                       status='OK',
                                       last_commit='',
                                       tags=None,
                                       ))
        ric.ensure_tags()
        self.assertEqual(ric.tags, tag1)
        ric.ensure_all_filled() # check that no exception is raised

        ric.ensure_branches_filled()
        self.assertEqual(ric.branches_filled, True)
        self.assertEqual(ric.branches_remote, [])
        self.assertEqual(ric.branches_local, [DEFAULT_BRANCH_NAME])



    def run_test_on_tag2_with_tag3(self, dir1, tag2: str, tag3: str):
        print('    - run_test_on_tag2_with_tag3')
        n, p, fp = str(dir1), dir1, str(dir1.resolve())
        # check refesh() effect
        ric = MgRepoInfo(n, fp)
        self.assertEqual(to_named_tuple(ric.refresh()),
                         RepoInfoTuple(name=n,
                                       head='tag tag2',
                                       tag='tag2',
                                       branch='',
                                       remote_synchro='',
                                       fullpath=fp,
                                       status='OK',
                                       last_commit='',
                                       tags=None,
                                       ))
        ric.ensure_tags()
        self.assertEqual(ric.tags, '%s %s' % (tag2, tag3))
        ric.ensure_all_filled() # check that no exception is raised

        ric.ensure_branches_filled()
        self.assertEqual(ric.branches_filled, True)
        self.assertEqual(ric.branches_remote, [])
        self.assertEqual(ric.branches_local, [DEFAULT_BRANCH_NAME])



    def run_test_on_commit(self, dir1, commit_sha1):
        print('    - run_test_on_commit')
        n, p, fp = str(dir1), dir1, str(dir1.resolve())
        # check refesh() effect
        ric = MgRepoInfo(n, fp)
        ric.refresh()
        self.assertEqual(to_named_tuple(ric),
                         RepoInfoTuple(name=n,
                                       head='commit %s' % commit_sha1,
                                       tag='',
                                       branch='',
                                       remote_synchro='',
                                       fullpath=fp,
                                       status='OK',
                                       last_commit=ric.last_commit,
                                       tags=None,
                                       ))
        self.assertTrue(ric.last_commit.startswith('commit %s' % commit_sha1))
        ric.ensure_tags()
        self.assertEqual(ric.tags, '')
        ric.ensure_all_filled()  # check that no exception is raised

        ric.ensure_branches_filled()
        self.assertEqual(ric.branches_filled, True)
        self.assertEqual(ric.branches_remote, [])
        self.assertEqual(ric.branches_local, [DEFAULT_BRANCH_NAME])



    def run_test_on_tag_like_sha1(self, dir1, tag1: str):
        print('    - run_test_on_tag_like_sha1')
        n, p, fp = str(dir1), dir1, str(dir1.resolve())
        # check refesh() effect
        ric = MgRepoInfo(n, fp)
        ric.refresh()
        self.assertEqual(to_named_tuple(ric),
                         RepoInfoTuple(name=n,
                                       head='tag %s' % tag1,
                                       tag=tag1,
                                       branch='',
                                       remote_synchro='',
                                       fullpath=fp,
                                       status='OK',
                                       last_commit=ric.last_commit,
                                       tags=None,
                                       ))
        ric.ensure_all_filled()  # check that no exception is raised

        ric.ensure_branches_filled()
        self.assertEqual(ric.branches_filled, True)
        self.assertEqual(ric.branches_local, [DEFAULT_BRANCH_NAME])
        self.assertEqual(ric.branches_remote, [])



    def run_test_on_other_branch(self, dir1, branch, _sha1):
        print('    - run_test_on_other_branch')
        n, p, fp = str(dir1), dir1, str(dir1.resolve())
        # check refesh() effect
        ric = MgRepoInfo(n, fp)
        ric.refresh()
        self.assertEqual(to_named_tuple(ric),
                         RepoInfoTuple(name=n,
                                       head='branch %s' % branch,
                                       tag='',
                                       branch=branch,
                                       remote_synchro=MSG_LOCAL_BRANCH,
                                       fullpath=fp,
                                       status='OK',
                                       last_commit=ric.last_commit,
                                       tags=None,
                                       ))
        ric.ensure_all_filled()  # check that no exception is raised

        ric.ensure_branches_filled()
        self.assertEqual(ric.branches_filled, True)
        self.assertEqual(ric.branches_remote, [])
        self.assertEqual(ric.branches_local, ['branch1', DEFAULT_BRANCH_NAME])



    def run_test_on_other_branch_merged_main(self, dir1, branch):
        print('    - run_test_on_other_branch_merged_main')
        n, p, fp = str(dir1), dir1, str(dir1.resolve())
        # check refesh() effect
        ric = MgRepoInfo(n, fp)
        ric.refresh()
        self.assertEqual(to_named_tuple(ric),
                         RepoInfoTuple(name=n,
                                       head='branch %s' % branch,
                                       tag='',
                                       branch=branch,
                                       remote_synchro=MSG_LOCAL_BRANCH,
                                       fullpath=fp,
                                       status='OK',
                                       last_commit=ric.last_commit,
                                       tags=None,
                                       ))
        ric.ensure_all_filled()  # check that no exception is raised

        ric.ensure_branches_filled()
        self.assertEqual(ric.branches_filled, True)
        self.assertEqual(ric.branches_remote, [])
        self.assertEqual(ric.branches_local, ['branch1', DEFAULT_BRANCH_NAME])


    def run_test_on_main_after_branch2_deleted(self, dir1, branch):
        print('    - run_test_on_main_after_branch2_deleted')
        n, p, fp = str(dir1), dir1, str(dir1.resolve())
        # check refesh() effect
        ric = MgRepoInfo(n, fp)
        ric.refresh()
        self.assertEqual(to_named_tuple(ric),
                         RepoInfoTuple(name=n,
                                       head='branch %s' % branch,
                                       tag='',
                                       branch=branch,
                                       remote_synchro=MSG_LOCAL_BRANCH,
                                       fullpath=fp,
                                       status='OK',
                                       last_commit=ric.last_commit,
                                       tags=None,
                                       ))
        ric.ensure_all_filled()  # check that no exception is raised

        ric.ensure_branches_filled()
        self.assertEqual(ric.branches_filled, True)
        self.assertEqual(ric.branches_remote, [])
        self.assertEqual(ric.branches_local, ['branch1', DEFAULT_BRANCH_NAME])



    def test_find_git_repos(self) -> None:
        self.dir2 = pathlib.Path(self.gitdir) / 'dir2'
        self.dir2.mkdir()
        os.chdir(self.dir2)
        git_init_repo('.')

        git_dir = self.gitdir
        os.chdir(git_dir)
        (git_dir/'tmp_a/.git').mkdir(parents=True)
        mr = MultiRepo(str(git_dir))
        repo_list = mr.find_git_repos()
        self.assertEqual(set(repo_list), {'dir2'})

        # nothing added, or removed
        added_repo, rm_repo = mr.find_git_repos_added_removed()
        self.assertEqual((added_repo, rm_repo), ([], []))

        # fake that 'dir2' is a new repo added and that 'removed' was removed
        removed_ric = MgRepoInfo('removed', str(git_dir / 'removed'))
        mr.repo_dict['removed'] = removed_ric
        mr.repo_names.append('removed')
        mr.repo_list.append(removed_ric)

        del mr.repo_dict['dir2']
        del mr.repo_names[0]
        del mr.repo_list[0]

        added_repo, rm_repo = mr.find_git_repos_added_removed()
        self.assertEqual(len(added_repo), 1)
        self.assertEqual(added_repo[0].name, 'dir2')
        self.assertEqual(rm_repo,    [mr.repo_dict['removed']])

        rmtree_failsafe(self.dir2)


    def test_find_git_repos_with_strange_names(self) -> None:
        strange_dir1 = pathlib.Path(self.gitdir) / 'strange_dir[with_bracket]'
        strange_dir1.mkdir()
        os.chdir(strange_dir1)
        git_init_repo(str(strange_dir1))

        strange_dir2 = pathlib.Path(strange_dir1) / 'strange_subdir[with_bracket]'
        strange_dir2.mkdir()
        git_init_repo(str(strange_dir2))

        os.chdir(self.gitdir)
        mr = MultiRepo(str(self.gitdir))
        repo_list = mr.find_git_repos()
        self.assertEqual(set(repo_list), {
            'strange_dir[with_bracket]', 
            os.path.join('strange_dir[with_bracket]', 'strange_subdir[with_bracket]')
        })

        os.chdir(strange_dir1)
        mr = MultiRepo(str(strange_dir1))
        repo_list = mr.find_git_repos()
        self.assertEqual(set(repo_list), {
            '.',
            'strange_subdir[with_bracket]'
        } )

        rmtree_failsafe(strange_dir1)


    def run_clone_test_on_empty_git(self, dir1: pathlib.Path, dir3: pathlib.Path):
        print('    - run_clone_test_on_empty_git')

        n, p, fp = str(dir3), dir3, str(dir3.resolve())
        git_exec('clone', str(dir1), fp)
        git_set_commit_author(str(dir3))

        # check refesh() effect
        ric = MgRepoInfo(n, fp)
        self.assertEqual(to_named_tuple(ric.refresh()),
                         RepoInfoTuple(name=n,
                                       head=MSG_EMPTY_REPO,
                                       branch=MSG_EMPTY_REPO,
                                       fullpath=fp,
                                       status='OK',
                                       last_commit=MSG_NO_COMMIT,
                                       tags='',
                                       ))
        self.assertEqual(ric.commit_sha1, '')
        ric.ensure_all_filled()  # check ensure_all_filled() does not fail

        ric.ensure_branches_filled()
        self.assertEqual(ric.branches_filled, True)
        self.assertEqual(ric.branches_remote, [])
        self.assertEqual(ric.branches_local, [])



    def run_clone_test_on_origin_main_2_commits(self, _dir1, dir3):
        print('    - run_clone_test_on_origin_main_2_commits')

        n, p, fp = str(dir3), dir3, str(dir3.resolve())

        # check refesh() effect
        ric = MgRepoInfo(n, fp)
        self.assertEqual(to_named_tuple(ric.refresh()),
                         RepoInfoTuple(name=n,
                                       head=MSG_EMPTY_REPO,
                                       branch=MSG_EMPTY_REPO,
                                       fullpath=fp,
                                       status='OK',
                                       last_commit=MSG_NO_COMMIT,
                                       tags='',
                                       ))

        git_exec('fetch', gitdir=dir3)
        self.assertEqual(to_named_tuple(ric.refresh()),
                         RepoInfoTuple(name=n,
                                       head=MSG_EMPTY_REPO,
                                       branch=MSG_EMPTY_REPO,
                                       fullpath=fp,
                                       status='OK',
                                       last_commit=MSG_NO_COMMIT,
                                       tags='',
                                       ))

        git_exec('pull', gitdir=dir3)
        self.assertEqual(to_named_tuple(ric.refresh()),
                         RepoInfoTuple(name=n,
                                       head=f'branch {DEFAULT_BRANCH_NAME}',
                                       branch=DEFAULT_BRANCH_NAME,
                                       fullpath=fp,
                                       status='OK',
                                       remote_synchro=MSG_REMOTE_SYNCHRO_OK,
                                       remote_branch=f'origin/{DEFAULT_BRANCH_NAME}',
                                       last_commit='',
                                       tags=None,
                                       ))

        ric.ensure_branches_filled()
        self.assertEqual(ric.branches_filled, True)
        self.assertEqual(ric.branches_local, [DEFAULT_BRANCH_NAME])
        self.assertEqual(ric.branches_remote, [f'origin/{DEFAULT_BRANCH_NAME}'])



    def run_clone_test_on_origin_main_more_commits(self, dir1, dir3):
        print('    - run_clone_test_on_origin_main_more_commits')

        n, p, fp = str(dir3), dir3, str(dir3.resolve())

        # check refesh() effect
        ric = MgRepoInfo(n, fp)
        self.assertEqual(to_named_tuple(ric.refresh()),
                         RepoInfoTuple(name=n,
                                       head=f'branch {DEFAULT_BRANCH_NAME}',
                                       branch=DEFAULT_BRANCH_NAME,
                                       fullpath=fp,
                                       status='OK',
                                       remote_synchro=MSG_REMOTE_SYNCHRO_OK,
                                       remote_branch=f'origin/{DEFAULT_BRANCH_NAME}',
                                       last_commit='',
                                       tags=None,
                                       ))
        git_exec('fetch', gitdir=dir3)
        self.assertEqual(to_named_tuple(ric.refresh()),
                         RepoInfoTuple(name=n,
                                       head=f'branch {DEFAULT_BRANCH_NAME}',
                                       branch=DEFAULT_BRANCH_NAME,
                                       fullpath=fp,
                                       status='OK',
                                       remote_synchro='2 to pull',
                                       remote_branch=f'origin/{DEFAULT_BRANCH_NAME}',
                                       last_commit='',
                                       tags=None,
                                       ))

        add_content(fp, 'blabla')
        git_commit(fp, 'add blabla', allow_errors=True) # when creating the commit, git returns 1 because the branches are diverging
        self.assertEqual(to_named_tuple(ric.refresh()),
                         RepoInfoTuple(name=n,
                                       head=f'branch {DEFAULT_BRANCH_NAME}',
                                       branch=DEFAULT_BRANCH_NAME,
                                       fullpath=fp,
                                       status='OK',
                                       remote_synchro='1 to push, 2 to pull',
                                       remote_branch=f'origin/{DEFAULT_BRANCH_NAME}',
                                       last_commit='',
                                       tags=None,
                                       ))

        ric.ensure_branches_filled()
        self.assertEqual(ric.branches_filled, True)
        self.assertEqual(ric.branches_local, [DEFAULT_BRANCH_NAME])
        self.assertEqual(ric.branches_remote, [f'origin/{DEFAULT_BRANCH_NAME}'])



    def run_clone_test_on_tag(self, dir1, tag, dir4, tag_sha1):
        # clone on a repo pointing on a tag
        print('    - run_clone_test_on_tag')

        n, p, fp = str(dir4), dir4, str(dir4.resolve())
        # clone on the tag, we are in detached state now
        git_exec('clone', str(dir1), str(dir4))
        git_set_commit_author(str(dir4))

        # check refesh() effect
        # the current status is that the fact that we are detached on a specific tag has not been
        # identified by git
        ric = MgRepoInfo(n, fp)
        self.assertEqual(to_named_tuple(ric.refresh()),
                         RepoInfoTuple(name=n,
                                       head='commit %s' % tag_sha1[:SHORT_SHA1_NB_DIGITS],
                                       branch='',
                                       tag='',
                                       fullpath=fp,
                                       status='OK',
                                       remote_synchro='',
                                       remote_branch='',
                                       last_commit=ric.last_commit,
                                       tags=None,
                                       ))


        sha1 = add_content(fp, 'blabla')
        self.assertEqual(to_named_tuple(ric.refresh()),
                         RepoInfoTuple(name=n,
                                       head='commit %s' % sha1,
                                       branch='',
                                       tag='',
                                       fullpath=fp,
                                       status='OK',
                                       remote_synchro='',
                                       remote_branch='',
                                       last_commit=ric.last_commit,
                                       tags=None,
                                       ))

        ric.ensure_branches_filled()
        self.assertEqual(ric.branches_filled, True)
        self.assertEqual(ric.branches_remote, [f'origin/{DEFAULT_BRANCH_NAME}'])
        self.assertEqual(ric.branches_local, [])



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'ui_git_exec_window.ui'
##
## Created by: Qt User Interface Compiler version 5.15.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *


class Ui_GitExecDialog(object):
    def setupUi(self, GitExecDialog: QDialog) -> None:
        if not GitExecDialog.objectName():
            GitExecDialog.setObjectName(u"GitExecDialog")
        GitExecDialog.resize(915, 624)
        self.verticalLayout = QVBoxLayout(GitExecDialog)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.treeGitJobs = QTreeWidget(GitExecDialog)
        __qtreewidgetitem = QTreeWidgetItem()
        __qtreewidgetitem.setText(0, u"col1");
        self.treeGitJobs.setHeaderItem(__qtreewidgetitem)
        QTreeWidgetItem(self.treeGitJobs)
        self.treeGitJobs.setObjectName(u"treeGitJobs")
        sizePolicy = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(3)
        sizePolicy.setHeightForWidth(self.treeGitJobs.sizePolicy().hasHeightForWidth())
        self.treeGitJobs.setSizePolicy(sizePolicy)
        self.treeGitJobs.setHeaderHidden(True)

        self.verticalLayout.addWidget(self.treeGitJobs)

        self.progressBar = QProgressBar(GitExecDialog)
        self.progressBar.setObjectName(u"progressBar")
        self.progressBar.setValue(24)
        self.progressBar.setTextVisible(False)
        self.progressBar.setInvertedAppearance(False)

        self.verticalLayout.addWidget(self.progressBar)

        self.widget = QWidget(GitExecDialog)
        self.widget.setObjectName(u"widget")

        self.verticalLayout.addWidget(self.widget)

        self.label = QLabel(GitExecDialog)
        self.label.setObjectName(u"label")
        font = QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label.setFont(font)

        self.verticalLayout.addWidget(self.label)

        self.textEditSummary = QTextEdit(GitExecDialog)
        self.textEditSummary.setObjectName(u"textEditSummary")
        sizePolicy1 = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Minimum)
        sizePolicy1.setHorizontalStretch(0)
        sizePolicy1.setVerticalStretch(0)
        sizePolicy1.setHeightForWidth(self.textEditSummary.sizePolicy().hasHeightForWidth())
        self.textEditSummary.setSizePolicy(sizePolicy1)
        self.textEditSummary.setReadOnly(True)

        self.verticalLayout.addWidget(self.textEditSummary)

        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.labelTasks = QLabel(GitExecDialog)
        self.labelTasks.setObjectName(u"labelTasks")

        self.horizontalLayout.addWidget(self.labelTasks)

        self.buttonBox = QDialogButtonBox(GitExecDialog)
        self.buttonBox.setObjectName(u"buttonBox")
        font1 = QFont()
        font1.setPointSize(10)
        self.buttonBox.setFont(font1)
        self.buttonBox.setOrientation(Qt.Horizontal)
        self.buttonBox.setStandardButtons(QDialogButtonBox.Discard|QDialogButtonBox.Ok)

        self.horizontalLayout.addWidget(self.buttonBox)


        self.verticalLayout.addLayout(self.horizontalLayout)


        self.retranslateUi(GitExecDialog)
        self.buttonBox.accepted.connect(GitExecDialog.accept)
        self.buttonBox.rejected.connect(GitExecDialog.reject)

        QMetaObject.connectSlotsByName(GitExecDialog)
    # setupUi

    def retranslateUi(self, GitExecDialog: QDialog) -> None:
        GitExecDialog.setWindowTitle(QCoreApplication.translate("GitExecDialog", u"Running Commands", None))
        ___qtreewidgetitem = self.treeGitJobs.headerItem()
        ___qtreewidgetitem.setText(1, QCoreApplication.translate("GitExecDialog", u"col2", None));

        __sortingEnabled = self.treeGitJobs.isSortingEnabled()
        self.treeGitJobs.setSortingEnabled(False)
        ___qtreewidgetitem1 = self.treeGitJobs.topLevelItem(0)
        ___qtreewidgetitem1.setText(1, QCoreApplication.translate("GitExecDialog", u"toto", None));
        ___qtreewidgetitem1.setText(0, QCoreApplication.translate("GitExecDialog", u"New Item", None));
        self.treeGitJobs.setSortingEnabled(__sortingEnabled)

        self.progressBar.setFormat(QCoreApplication.translate("GitExecDialog", u"%v of %m done", None))
        self.label.setText(QCoreApplication.translate("GitExecDialog", u"Access right summary :", None))
        self.labelTasks.setText(QCoreApplication.translate("GitExecDialog", u"TextLabel", None))
    # retranslateUi

